#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import JAN, FEB, MAR, APR, JUN, SEP, NOV
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON, SUN_TO_NEXT_TUE


class PapuaNewGuinea(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """Papua New Guinea holidays.

    References:
        * [Public Holidays Act 1953](https://web.archive.org/web/20250406104407/http://www.paclii.org/pg/legis/consol_act/pha1953163.pdf)
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Papua_New_Guinea>
        * <https://web.archive.org/web/20240612224551/https://pnghausbung.com/kings-birthday-holiday-to-be-observed-on-16th-june/>

    Checked With:
        * <https://web.archive.org/web/20250215234848/https://www.ipa.gov.pg/public/Holidays.aspx>
        * <https://web.archive.org/web/20240119163205/https://www.businessadvantagepng.com/papua-new-guinea-public-for-holidays/>
        * [2019](https://web.archive.org/web/20250429081247/https://www.scribd.com/document/465334129/PNG-2019-Gazetted-Public-Holidays-pdf)
        * [2020](https://web.archive.org/web/20250429081229/https://publicholidays.asia/wp-content/uploads/2020/01/PNG_PublicHolidays_2020.png)
        * [2021](https://web.archive.org/web/20240625200332/https://publicholidays.asia/wp-content/uploads/2020/12/PNG_PublicHolidays_2021.pdf)
        * [2022](https://web.archive.org/web/20230530042019/https://publicholidays.asia/wp-content/uploads/2022/01/PNG_PublicHolidays_2022.pdf)
        * [2023](https://web.archive.org/web/20240413053729/https://publicholidays.asia/wp-content/uploads/2022/12/PNG_PublicHolidays_2023.pdf)

    Should a holiday listed, other than the Christmas Day, fall on a Sunday the next Monday
    shall, unless the Head of State, acting on advice, declares otherwise, be observed as a
    public holiday throughout the country.

    When Christmas Day falls on a Sunday, the following Tuesday shall also be observed as a
    public holiday.
    """

    country = "PG"
    observed_label = "%s (observed)"
    # Public Holidays Law 1953 (No. 38 of 1953).
    start_year = 1953

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, PapuaNewGuineaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Section 1: Public Holidays.
        # - Easter Saturday is currently not gazetted as a Public Holiday in 2024.
        # - While Easter Sunday itself is not listed, this is de facto always a day-off.

        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Good Friday.
        self._add_good_friday("Good Friday")

        if self._year <= 2023:
            # Easter Saturday.
            self._add_holy_saturday("Easter Saturday")

        # Easter Sunday.
        self._add_easter_sunday("Easter Sunday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Papua New Guinea Remembrance Day.
        self._add_observed(self._add_holiday_jul_23("Papua New Guinea Remembrance Day"))

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SUN_TO_NEXT_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two("Boxing Day"))

        # Section 2: Independence Day or Days.

        # Independence Day
        # Commemorates the attainment of Independent Sovereign Statehood on Sep 16, 1975.

        if self._year >= 1975:
            # Independence Day.
            self._add_observed(self._add_holiday_sep_16("Independence Day"))

        # Section 3: Anniversary of Birthday of the Queen/King.
        # In Papua New Guinea, it is usually celebrated on the second Monday of June every year.
        # From 2023, the date changed to Jun 17 (Special Observed for 2023 on Jun 16).

        if self._year <= 2022:
            # Queen's Birthday.
            self._add_holiday_2nd_mon_of_jun("Queen's Birthday")
        else:
            # King's Birthday.
            self._add_holiday_jun_17("King's Birthday")

        # Section 5: Additional Public Holidays.

        # Grand Chief Sir Michael Somare Remembrance Day.
        # First observed on Feb 26, 2022.
        # As of 2024, all previous observances so far are done on an ad hoc basis.

        if self._year >= 2022:
            # Grand Chief Sir Michael Somare Remembrance Day.
            self._add_holiday_feb_26("Grand Chief Sir Michael Somare Remembrance Day")

        # National Repentance Day.
        # Enacted Aug 15, 2011, celebrated by "prayer ceremonies" across the country.
        # First observed on Aug 26, 2011, held annually since then on that date.

        if self._year >= 2011:
            # The National Repentance Day.
            self._add_observed(self._add_holiday_aug_26("National Repentance Day"))


class PG(PapuaNewGuinea):
    pass


class PNG(PapuaNewGuinea):
    pass


class PapuaNewGuineaStaticHolidays:
    """Papua New Guinea special holidays.

    References:
        * [2018 APEC Public Holiday](https://web.archive.org/web/20220813203130/https://www.rnz.co.nz/international/pacific-news/369989/papua-new-guinea-declares-apec-holiday)
        * [2021 Sir Mekere Morauta's Funeral](https://web.archive.org/web/20241206065336/https://pnghausbung.com/friday-8th-declared-a-nation-wide-public-holiday-to-honour-late-sir-mekere/)
        * [2021 Sir Michael Somare's Day of Mourning](https://web.archive.org/web/20241207121149/https://pnghausbung.com/2-weeks-mourning-period-for-late-sir-micheal-to-start-with-public-holiday-on-monday-pm/)
        * [2022 QE2's Funeral](https://web.archive.org/web/20220914111729/https://www.thenational.com.pg/column-1-553/)
        * [2023 Sir Rabbie Namaliu's Funeral](https://web.archive.org/web/20241109201443/https://pnghausbung.com/breaking-public-holiday-tomorrow-in-respect-of-late-sir-rabbies-state-funeral/)

    Nov 15, 2018 is an APEC Holiday too, but for Port Moresby-only:
    <https://web.archive.org/web/20250120182905/https://www.businessadvantagepng.com/public-holidays-declared-in-port-moresby-due-to-apec-summit-says-ceo/>

    Starting from 2021 afterwards all state funerals are given special public holidays, though only
    some are day-off nationwide.
    """

    # National Day of Mourning for Sir Michael Somare.
    sir_michael_somare_mourning_day = "National Day of Mourning for Sir Michael Somare"

    # Grand Chief Sir Michael Somare Remembrance Day
    sir_michael_somare_remembrance_day = "Grand Chief Sir Michael Somare Remembrance Day"

    special_public_holidays = {
        # APEC Leaders' Summit Public Holiday.
        2018: (NOV, 16, "APEC Leaders' Summit Public Holiday"),
        2021: (
            # State Funeral of Sir Mekere Morauta.
            (JAN, 8, "State Funeral of Sir Mekere Morauta"),
            (MAR, 1, sir_michael_somare_mourning_day),
            (MAR, 12, sir_michael_somare_mourning_day),
        ),
        # State Funeral of Queen Elizabeth II.
        2022: (SEP, 19, "State Funeral of Queen Elizabeth II"),
        # State Funeral of Sir Rabbie Namaliu.
        2023: (APR, 18, "State Funeral of Sir Rabbie Namaliu"),
    }

    special_public_holidays_observed = {
        2022: (FEB, 28, sir_michael_somare_remembrance_day),
        2023: (
            (FEB, 24, sir_michael_somare_remembrance_day),
            (JUN, 16, "King's Birthday"),
            (SEP, 15, "Independence Day"),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

__all__ = (
    "country_holidays",
    "CountryHoliday",
    "financial_holidays",
    "list_localized_countries",
    "list_localized_financial",
    "list_long_breaks",
    "list_supported_countries",
    "list_supported_financial",
)

import warnings
from collections.abc import Iterable
from datetime import date
from functools import cache

from holidays.calendars.gregorian import _timedelta
from holidays.holiday_base import CategoryArg, HolidayBase
from holidays.registry import EntityLoader


def country_holidays(
    country: str,
    subdiv: str | None = None,
    years: int | Iterable[int] | None = None,
    expand: bool = True,
    observed: bool = True,
    prov: str | None = None,
    state: str | None = None,
    language: str | None = None,
    categories: CategoryArg | None = None,
) -> HolidayBase:
    """Return a new dictionary-like [`HolidayBase`][holidays.holiday_base.HolidayBase] object.

    Include public holidays for the country matching `country` and other keyword arguments.

    Args:
        country:
            An ISO 3166-1 Alpha-2 country code.

        subdiv:
            The subdivision (e.g. state or province) as a ISO 3166-2 code
            or its alias; not implemented for all countries (see documentation).

        years:
            The year(s) to pre-calculate public holidays for at instantiation.

        expand:
            Whether the entire year is calculated when one date from that year
            is requested.

        observed:
            Whether to include the dates of when public holidays are observed
            (e.g. a holiday falling on a Sunday being observed the following
            Monday). `False` may not work for all countries.

        prov:
            *deprecated* use `subdiv` instead.

        state:
            *deprecated* use `subdiv` instead.

        language:
            Specifies the language in which holiday names are returned.

            Accepts either:

            * A two-letter ISO 639-1 language code (e.g., 'en' for English, 'fr' for French),
                or
            * A language and entity combination using an underscore (e.g., 'en_US' for U.S.
                English, 'pt_BR' for Brazilian Portuguese).

            !!! warning
                The provided language or locale code must be supported by the holiday
                entity. Unsupported values will result in names being shown in the entity's
                original language.

            If not explicitly set (`language=None`), the system attempts to infer the
            language from the environment's locale settings. The following environment
            variables are checked, in order of precedence: LANGUAGE, LC_ALL, LC_MESSAGES, LANG.

            If none of these are set or they are empty, holiday names will default to the
            original language of the entity's holiday implementation.

            !!! warning
                This fallback mechanism may yield inconsistent results across environments
                (e.g., between a terminal session and a Jupyter notebook).

            To ensure consistent behavior, it is recommended to set the language parameter
            explicitly. If the specified language is not supported, holiday names will remain
            in the original language of the entity's holiday implementation.

            This behavior will be updated and formalized in v1.

        categories:
            Requested holiday categories.

    Returns:
        A [`HolidayBase`][holidays.holiday_base.HolidayBase] object matching the `country`.

    The key of the `dict`-like [`HolidayBase`][holidays.holiday_base.HolidayBase]
    object is the `date` of the holiday, and the value is the name of the holiday itself.
    Dates where a key is not present are not public holidays (or, if
    `observed` is `False`, days when a public holiday is observed).

    When passing the `date` as a key, the `date` can be expressed in one of the
    following types:

    * `datetime.date`,
    * `datetime.datetime`,
    * a `str` of any format recognized by `dateutil.parser.parse()`,
    * or a `float` or `int` representing a POSIX timestamp.

    The key is always returned as a `datetime.date` object.

    To maximize speed, the list of public holidays is built on the fly as
    needed, one calendar year at a time. When the object is instantiated
    without a `years` parameter, it is empty, but, unless `expand` is set
    to `False`, as soon as a key is accessed the class will calculate that entire
    year's list of holidays and set the keys with them.

    If you need to list the holidays as opposed to querying individual dates,
    instantiate the class with the `years` parameter.

    Example usage:

        >>> from holidays import country_holidays
        >>> us_holidays = country_holidays('US')
        # For a specific subdivision (e.g. state or province):
        >>> calif_holidays = country_holidays('US', subdiv='CA')

    The below will cause 2015 holidays to be calculated on the fly:

        >>> from datetime import date
        >>> assert date(2015, 1, 1) in us_holidays

    This will be faster because 2015 holidays are already calculated:

        >>> assert date(2015, 1, 2) not in us_holidays

    The [`HolidayBase`][holidays.holiday_base.HolidayBase] class also recognizes
    strings of many formats and numbers representing a POSIX timestamp:

        >>> assert '2014-01-01' in us_holidays
        >>> assert '1/1/2014' in us_holidays
        >>> assert 1388597445 in us_holidays

    Show the holiday's name:

        >>> us_holidays.get('2014-01-01')
        "New Year's Day"

    Check a range:

        >>> us_holidays['2014-01-01': '2014-01-03']
        [datetime.date(2014, 1, 1)]

    List all 2020 holidays:

        >>> us_holidays = country_holidays('US', years=2020)
        >>> for day in sorted(us_holidays.items()):
        ...     print(day)
        (datetime.date(2020, 1, 1), "New Year's Day")
        (datetime.date(2020, 1, 20), 'Martin Luther King Jr. Day')
        (datetime.date(2020, 2, 17), "Washington's Birthday")
        (datetime.date(2020, 5, 25), 'Memorial Day')
        (datetime.date(2020, 7, 3), 'Independence Day (observed)')
        (datetime.date(2020, 7, 4), 'Independence Day')
        (datetime.date(2020, 9, 7), 'Labor Day')
        (datetime.date(2020, 10, 12), 'Columbus Day')
        (datetime.date(2020, 11, 11), 'Veterans Day')
        (datetime.date(2020, 11, 26), 'Thanksgiving Day')
        (datetime.date(2020, 12, 25), 'Christmas Day')

    Some holidays are only present in parts of a country:

        >>> us_pr_holidays = country_holidays('US', subdiv='PR')
        >>> assert '2018-01-06' not in us_holidays
        >>> assert '2018-01-06' in us_pr_holidays

    Append custom holiday dates by passing one of:

    * a `dict` with date/name key/value pairs (e.g.
      `{'2010-07-10': 'My birthday!'}`),
    * a list of dates (as a `datetime.date`, `datetime.datetime`,
      `str`, `int`, or `float`); "Holiday" will be used as a description,
    * or a single date item (of one of the types above); "Holiday" will be
      used as a description:

    ```python
    >>> custom_holidays = country_holidays('US', years=2015)
    >>> custom_holidays.update({'2015-01-01': "New Year's Day"})
    >>> custom_holidays.update(['2015-07-01', '07/04/2015'])
    >>> custom_holidays.update(date(2015, 12, 25))
    >>> assert date(2015, 1, 1) in custom_holidays
    >>> assert date(2015, 1, 2) not in custom_holidays
    >>> assert '12/25/2015' in custom_holidays
    ```

    For more complex logic, like 4th Monday of January, you can inherit the
    [`HolidayBase`][holidays.holiday_base.HolidayBase] class and define your
    own `_populate` method. See documentation for examples.
    """
    import holidays

    try:
        return getattr(holidays, country)(
            years=years,
            subdiv=subdiv,
            expand=expand,
            observed=observed,
            prov=prov,
            state=state,
            language=language,
            categories=categories,
        )
    except AttributeError:
        raise NotImplementedError(f"Country {country} not available")


def financial_holidays(
    market: str,
    subdiv: str | None = None,
    years: int | Iterable[int] | None = None,
    expand: bool = True,
    observed: bool = True,
    language: str | None = None,
    categories: CategoryArg | None = None,
) -> HolidayBase:
    """Return a new dictionary-like [`HolidayBase`][holidays.holiday_base.HolidayBase] object.

    Include public holidays for the financial market matching `market` and other keyword
    arguments.

    Args:
        market:
            An ISO 10383 MIC code.

        subdiv:
            Currently not implemented for markets (see documentation).

        years:
            The year(s) to pre-calculate public holidays for at instantiation.

        expand:
            Whether the entire year is calculated when one date from that year
            is requested.

        observed:
            Whether to include the dates of when public holidays are observed
            (e.g. a holiday falling on a Sunday being observed the following
            Monday). `False` may not work for all markets.

        language:
            Specifies the language in which holiday names are returned.

            Accepts either:

            * A two-letter ISO 639-1 language code (e.g., 'en' for English, 'fr' for French),
                or
            * A language and entity combination using an underscore (e.g., 'en_US' for U.S.
                English, 'pt_BR' for Brazilian Portuguese).

            !!! warning
                The provided language or locale code must be supported by the holiday
                entity. Unsupported values will result in names being shown in the entity's
                original language.

            If not explicitly set (`language=None`), the system attempts to infer the
            language from the environment's locale settings. The following environment
            variables are checked, in order of precedence: LANGUAGE, LC_ALL, LC_MESSAGES, LANG.

            If none of these are set or they are empty, holiday names will default to the
            original language of the entity's holiday implementation.

            !!! warning
                This fallback mechanism may yield inconsistent results across environments
                (e.g., between a terminal session and a Jupyter notebook).

            To ensure consistent behavior, it is recommended to set the language parameter
            explicitly. If the specified language is not supported, holiday names will remain
            in the original language of the entity's holiday implementation.

            This behavior will be updated and formalized in v1.

        categories:
            Requested holiday categories.

    Returns:
        A [`HolidayBase`][holidays.holiday_base.HolidayBase] object matching the `market`.

    Example usage:

        >>> from holidays import financial_holidays
        >>> nyse_holidays = financial_holidays('XNYS')

    See [country_holidays()][holidays.utils.country_holidays] documentation for further
    details and examples.
    """
    import holidays

    try:
        return getattr(holidays, market)(
            years=years,
            subdiv=subdiv,
            expand=expand,
            observed=observed,
            language=language,
            categories=categories,
        )
    except AttributeError:
        raise NotImplementedError(f"Financial market {market} not available")


def CountryHoliday(  # noqa: N802
    country: str,
    subdiv: str | None = None,
    years: int | Iterable[int] | None = None,
    expand: bool = True,
    observed: bool = True,
    prov: str | None = None,
    state: str | None = None,
) -> HolidayBase:
    """
    Note:
        Deprecated name for `country_holidays()`.
    """

    warnings.warn(
        "CountryHoliday is deprecated, use country_holidays instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    return country_holidays(country, subdiv, years, expand, observed, prov, state)


def _list_localized_entities(entity_codes: Iterable[str]) -> dict[str, list[str]]:
    """Get all localized entities and languages they support.

    Args:
        entity_codes:
            A list of entity codes.

    Returns:
        A dictionary where key is an entity code and value is a list of supported
        languages (either ISO 639-1 or a combination of ISO 639-1 and ISO 3166-1 codes joined
        with "_").
    """
    import holidays

    localized_countries = {}
    for entity_code in entity_codes:
        languages = getattr(holidays, entity_code).supported_languages
        if not languages:
            continue
        localized_countries[entity_code] = sorted(languages)

    return localized_countries


@cache
def list_localized_countries(include_aliases: bool = True) -> dict[str, list[str]]:
    """Get all localized countries and languages they support.

    Args:
        include_aliases:
            Whether to include entity aliases (e.g. UK for GB).

    Returns:
        A dictionary where key is an ISO 3166-1 alpha-2 country code and value is a
        list of supported languages (either ISO 639-1 or a combination of ISO 639-1
        and ISO 3166-1 codes joined with "_").
    """
    return _list_localized_entities(EntityLoader.get_country_codes(include_aliases))


@cache
def list_localized_financial(include_aliases: bool = True) -> dict[str, list[str]]:
    """Get all localized financial markets and languages they support.

    Args:
        include_aliases:
            Whether to include entity aliases (e.g. TAR for ECB, XNYS for NYSE).

    Returns:
        A dictionary where key is a market code and value is a list of supported
        subdivision codes.
    """
    return _list_localized_entities(EntityLoader.get_financial_codes(include_aliases))


def _list_supported_entities(entity_codes: Iterable[str]) -> dict[str, list[str]]:
    """Get all supported entities and their subdivisions.

    Args:
        entity_codes:
            A list of entity codes.

    Returns:
        A dictionary where key is an entity code and value is a list of supported
        subdivision codes.
    """
    import holidays

    return {
        country_code: list(getattr(holidays, country_code).subdivisions)
        for country_code in entity_codes
    }


@cache
def list_supported_countries(include_aliases: bool = True) -> dict[str, list[str]]:
    """Get all supported countries and their subdivisions.

    Args:
        include_aliases:
            Whether to include entity aliases (e.g. UK for GB).

    Returns:
        A dictionary where key is an ISO 3166-1 alpha-2 country code and value
        is a list of supported subdivision codes.
    """
    return _list_supported_entities(EntityLoader.get_country_codes(include_aliases))


@cache
def list_supported_financial(include_aliases: bool = True) -> dict[str, list[str]]:
    """Get all supported financial markets and their subdivisions.

    Args:
        include_aliases:
            Whether to include entity aliases (e.g. NYSE for XNYS, TAR for XECB).

    Returns:
        A dictionary where key is a market code and value is a list of supported
        subdivision codes.
    """
    return _list_supported_entities(EntityLoader.get_financial_codes(include_aliases))


def list_long_breaks(
    instance: HolidayBase, *, minimum_break_length: int = 3, require_weekend_overlap: bool = True
) -> list[list[date]]:
    """Get consecutive holidays.

    Args:
        instance:
            [`HolidayBase`][holidays.holiday_base.HolidayBase] object containing holidays data.

        minimum_break_length:
            The minimum number of consecutive holidays required for a break period
            to be considered a long one. Defaults to 3.

        require_weekend_overlap:
            Whether to include only consecutive holidays that overlap with a weekend.
            Defaults to True.

    Returns:
        A list of consecutive holidays longer than or equal to the specified minimum length.
    """
    long_breaks = []
    seen_dates = set()

    for dt in sorted(instance.keys()):
        if dt in seen_dates:
            continue

        previous_working_day = instance.get_nth_working_day(dt, -1)
        next_working_day = instance.get_nth_working_day(dt, +1)
        long_break_length = (next_working_day - previous_working_day).days - 1

        if long_break_length < minimum_break_length:
            continue

        is_long_break = not require_weekend_overlap
        long_break_dates = []
        for delta_days in range(1, long_break_length + 1):
            holiday = _timedelta(previous_working_day, delta_days)
            if not is_long_break and instance._is_weekend(holiday):
                is_long_break = True
            long_break_dates.append(holiday)
            seen_dates.add(holiday)

        if is_long_break:
            long_breaks.append(long_break_dates)

    return long_breaks



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import (
    _CustomBuddhistHolidays,
    _CustomChineseHolidays,
    _CustomIslamicHolidays,
)
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.constants import GOVERNMENT, PUBLIC
from holidays.groups import (
    BalineseSakaCalendarHolidays,
    BuddhistCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase


class Indonesia(
    ObservedHolidayBase,
    BalineseSakaCalendarHolidays,
    BuddhistCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Indonesia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Indonesia>
        * <https://id.wikipedia.org/wiki/Hari_libur_di_Indonesia>
        * <https://web.archive.org/web/20250413192412/https://www.liburnasional.com/sejarah/>
        * [1963-2025](https://id.wikipedia.org/wiki/Indonesia_dalam_tahun_1963)
        * <https://web.archive.org/web/20250413192423/https://www.timeanddate.com/holidays/indonesia/>
    """

    country = "ID"
    default_language = "id"
    # %s (estimated).
    estimated_label = tr("%s (perkiraan)")
    # %s (observed).
    observed_label = tr("Pegangti %s")
    # %s (observed, estimated).
    observed_estimated_label = tr("Pegangti %s (perkiraan)")
    supported_languages = ("en_US", "id", "th", "uk")
    supported_categories = (GOVERNMENT, PUBLIC)
    start_year = 1946

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        BalineseSakaCalendarHolidays.__init__(self)
        BuddhistCalendarHolidays.__init__(self, cls=IndonesiaBuddhistHolidays, show_estimated=True)
        ChineseCalendarHolidays.__init__(self, cls=IndonesiaChineseHolidays, show_estimated=True)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=IndonesiaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, cls=IndonesiaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Tahun Baru Masehi"))

        # Independence Day.
        self._add_holiday_aug_17(tr("Hari Kemerdekaan Republik Indonesia"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Removed)
        if self._year <= 1952:
            # Armed Forces Day.
            self._add_holiday_oct_5(tr("Hari Angkatan Perang"))

            # Heroes' Day.
            self._add_holiday_nov_10(tr("Hari Pahlawan"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Added Nationally)
        if self._year >= 1953:
            # Christmas Day.
            self._add_christmas_day(tr("Hari Raya Natal"))

            # Eid al-Fitr.
            self._add_eid_al_fitr_day(tr("Hari Raya Idul Fitri"))

            # Eid al-Fitr Second Day.
            self._add_eid_al_fitr_day_two(tr("Hari kedua dari Hari Raya Idul Fitri"))

            # Eid al-Adha.
            self._add_eid_al_adha_day(tr("Hari Raya Idul Adha"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Added Nationally)
        # Keputusan Presiden no 21 tahun 1963. (1963-10-15; Moved to Denominative-based)
        # Keputusan Presiden no 251 tahun 1967. (1967-12-16; Removed)
        if 1953 <= self._year <= 1963:
            # Easter Monday.
            self._add_easter_monday(tr("Hari kedua Paskah"))

            # Whit Monday.
            self._add_whit_monday(tr("Hari kedua Pentakosta"))

            # Nuzul Al Quran.
            self._add_nuzul_al_quran_day(tr("Nuzululqur'an"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Added Nationally)
        # Keputusan Presiden no 21 tahun 1963. (1963-10-15; Moved to Denominative-based)
        # Keputusan Presiden no 251 tahun 1967. (1967-12-16; Added Nationally)
        if 1953 <= self._year <= 1962 or self._year >= 1968:
            # Isra' and Mi'raj.
            self._add_isra_and_miraj_day(tr("Isra Mikraj Nabi Muhammad"))

        # (Same as above, was before 1963 cut-off date)
        if 1953 <= self._year <= 1963 or self._year >= 1968:
            # Ascension Day.
            self._add_ascension_thursday(tr("Kenaikan Yesus Kristus"))

            # Islamic New Year.
            self._add_islamic_new_year_day(tr("Tahun Baru Islam"))

            # Prophet's Birthday.
            self._add_mawlid_day(tr("Maulid Nabi Muhammad"))

        # Keputusan Presiden no 21 tahun 1963. (1963-10-15; Added Denominative-based)
        # Keputusan Presiden no 251 tahun 1967. (1967-12-16; Added Nationally)
        # Keputusan Presiden no 10 tahun 1971. (1971-03-15; Removed)
        if 1968 <= self._year <= 1970:
            # Assumption Day.
            self._add_assumption_of_mary_day(tr("Mikraj Santa Maria"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Added Nationally)
        # Keputusan Presiden no 21 tahun 1963. (1963-10-15; Moved to Denominative-based)
        # Keputusan Presiden no 251 tahun 1967. (1967-12-16; Removed)
        # Keputusan Presiden no 10 tahun 1971. (1971-03-15; Added Nationally)
        if 1953 <= self._year <= 1963 or self._year >= 1971:
            # Good Friday.
            self._add_good_friday(tr("Wafat Yesus Kristus"))

        # Keputusan Presiden no 3 tahun 1983. (1983-01-19; Added Nationally)
        if self._year >= 1983:
            # Day of Silence.
            self._add_nyepi(tr("Hari Suci Nyepi"))

            # Vesak Day.
            self._add_vesak(tr("Hari Raya Waisak"))

        # Keputusan Presiden no 19 tahun 2002. (2002-04-09; Added Nationally)
        if self._year >= 2003:
            # Lunar New Year.
            self._add_chinese_new_years_day(tr("Tahun Baru Imlek"))

        # Keputusan Presiden no 24 tahun 1953. (1953-01-01; Added Nationally)
        # Keputusan Presiden no 148 tahun 1968. (1968-04-18; Removed)
        # Keputusan Presiden no 24 tahun 2013. (2013-07-29; Added Nationally)
        if 1953 <= self._year <= 1967 or self._year >= 2014:
            # International Labor Day.
            self._add_labor_day(tr("Hari Buruh Internasional"))

        # Keputusan Presiden no 24 tahun 2016. (2016-06-01; Added Nationally)
        if self._year >= 2016:
            # Pancasila Day.
            self._add_holiday_jun_1(tr("Hari Lahir Pancasila"))

        # Keputusan Presiden no 8 tahun 2024. (2024-01-29; Added Nationally)
        # This KEPPRES overwrites all pre-existing ones.
        if self._year >= 2024:
            # Easter Sunday.
            self._add_easter_sunday(tr("Kebangkitan Yesus Kristus"))


class ID(Indonesia):
    pass


class IDN(Indonesia):
    pass


class IndonesiaBuddhistHolidays(_CustomBuddhistHolidays):
    VESAK_DATES = {
        1983: (MAY, 27),
        1984: (MAY, 14),
        1985: (JUN, 3),
        1986: (MAY, 24),
        1987: (MAY, 13),
        1988: (MAY, 31),
        1989: (MAY, 21),
        1990: (MAY, 10),
        1991: (MAY, 28),
        1992: (MAY, 16),
        1993: (MAY, 6),
        1994: (MAY, 25),
        1995: (MAY, 15),
        1996: (JUN, 2),
        1997: (MAY, 22),
        1998: (MAY, 11),
        1999: (MAY, 30),
        2000: (MAY, 18),
        2001: (MAY, 7),
        2002: (MAY, 26),
        2003: (MAY, 16),
        2004: (JUN, 3),
        2005: (MAY, 24),
        2006: (MAY, 13),
        2007: (JUN, 1),
        2008: (MAY, 20),
        2009: (MAY, 9),
        2010: (MAY, 28),
        2011: (MAY, 17),
        2012: (MAY, 6),
        2013: (MAY, 25),
        2014: (MAY, 15),
        2015: (JUN, 2),
        2016: (MAY, 22),
        2017: (MAY, 11),
        2018: (MAY, 29),
        2019: (MAY, 19),
        2020: (MAY, 7),
        2021: (MAY, 26),
        2022: (MAY, 16),
        2023: (JUN, 4),
        2024: (MAY, 23),
        2025: (MAY, 12),
    }


class IndonesiaChineseHolidays(_CustomChineseHolidays):
    LUNAR_NEW_YEAR_DATES_CONFIRMED_YEARS = (2003, 2025)
    LUNAR_NEW_YEAR_DATES = {
        2006: (JAN, 30),
        2007: (FEB, 19),
        2010: (FEB, 15),
        2013: (FEB, 11),
    }


class IndonesiaIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (1963, 2025)
    EID_AL_ADHA_DATES = {
        1963: (MAY, 4),
        1964: (APR, 23),
        1965: (APR, 12),
        1966: (APR, 2),
        1967: (MAR, 22),
        1970: (FEB, 17),
        1972: (JAN, 27),
        1973: (JAN, 15),
        1974: ((JAN, 4), (DEC, 24)),
        1976: (DEC, 2),
        1978: (NOV, 11),
        1982: (SEP, 28),
        1984: (SEP, 6),
        1986: (AUG, 16),
        1987: (AUG, 5),
        1988: (JUL, 24),
        1990: (JUL, 3),
        1991: (JUN, 23),
        1993: (JUN, 1),
        1994: (MAY, 21),
        1995: (MAY, 10),
        1996: (APR, 28),
        1997: (APR, 18),
        1999: (MAR, 28),
        2002: (FEB, 23),
        2003: (FEB, 12),
        2004: (FEB, 2),
        2010: (NOV, 17),
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 12),
        2018: (AUG, 22),
        2022: (JUL, 10),
        2023: (JUN, 29),
        2024: (JUN, 17),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (1963, 2025)
    EID_AL_FITR_DATES = {
        1963: (FEB, 25),
        1964: (FEB, 15),
        1965: (FEB, 3),
        1966: (JAN, 23),
        1968: ((JAN, 2), (DEC, 21)),
        1969: (DEC, 11),
        1974: (OCT, 17),
        1976: (SEP, 25),
        1977: (SEP, 15),
        1978: (SEP, 4),
        1979: (AUG, 24),
        1982: (JUL, 22),
        1983: (JUL, 12),
        1985: (JUN, 20),
        1986: (JUN, 9),
        1987: (MAY, 29),
        1988: (MAY, 17),
        1989: (MAY, 7),
        1991: (APR, 16),
        1992: (APR, 5),
        1993: (MAR, 25),
        1994: (MAR, 14),
        1995: (MAR, 3),
        1996: (FEB, 20),
        1997: (FEB, 9),
        1998: (JAN, 30),
        1999: (JAN, 19),
        2002: (DEC, 6),
        2006: (OCT, 24),
        2019: (JUN, 5),
        2023: (APR, 22),
        2025: (MAR, 31),
    }

    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (1968, 2025)
    HIJRI_NEW_YEAR_DATES = {
        1970: (MAR, 10),
        1971: (FEB, 27),
        1973: (FEB, 5),
        1974: (JAN, 25),
        1975: (JAN, 14),
        1976: ((JAN, 3), (DEC, 22)),
        1977: (DEC, 12),
        1979: (NOV, 21),
        1981: (OCT, 29),
        1983: (OCT, 8),
        1985: (SEP, 16),
        1987: (AUG, 26),
        1988: (AUG, 14),
        1989: (AUG, 3),
        1991: (JUL, 13),
        1992: (JUL, 2),
        1994: (JUN, 11),
        1995: (MAY, 31),
        1996: (MAY, 19),
        1997: (MAY, 8),
        1998: (APR, 28),
        2003: (MAR, 3),
        2004: (FEB, 23),
        2011: (NOV, 27),
        2013: (NOV, 5),
        2019: (SEP, 1),
        2021: (AUG, 11),
        2025: (JUN, 27),
    }

    ISRA_AND_MIRAJ_DATES_CONFIRMED_YEARS = (1968, 2025)
    ISRA_AND_MIRAJ_DATES = {
        1968: (OCT, 20),
        1969: (OCT, 9),
        1970: (SEP, 29),
        1972: (SEP, 6),
        1973: (AUG, 26),
        1974: (AUG, 16),
        1976: (JUL, 25),
        1977: (JUL, 14),
        1978: (JUL, 3),
        1980: (JUN, 11),
        1982: (MAY, 21),
        1984: (APR, 29),
        1985: (APR, 18),
        1986: (APR, 7),
        1988: (MAR, 16),
        1990: (FEB, 23),
        1991: (FEB, 12),
        1992: (FEB, 1),
        1994: ((JAN, 10), (DEC, 30)),
        1995: (DEC, 20),
        1997: (NOV, 28),
        1998: (NOV, 17),
        1999: (NOV, 6),
        2000: (OCT, 25),
        2001: (OCT, 15),
        2003: (SEP, 22),
        2004: (SEP, 13),
        2005: (SEP, 2),
        2007: (AUG, 11),
        2010: (JUL, 10),
        2014: (MAY, 27),
        2016: (MAY, 6),
        2018: (APR, 14),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (1968, 2025)
    MAWLID_DATES = {
        1972: (APR, 26),
        1973: (APR, 14),
        1974: (APR, 5),
        1975: (MAR, 25),
        1976: (MAR, 13),
        1978: (FEB, 20),
        1982: ((JAN, 8), (DEC, 28)),
        1983: (DEC, 17),
        1984: (DEC, 5),
        1985: (NOV, 25),
        1987: (NOV, 4),
        1988: (OCT, 23),
        1989: (OCT, 12),
        1991: (SEP, 21),
        1993: (AUG, 30),
        1994: (AUG, 20),
        1995: (AUG, 9),
        1996: (JUL, 28),
        1997: (JUL, 17),
        2000: (JUN, 15),
        2002: (MAY, 25),
        2003: (MAY, 15),
        2004: (MAY, 3),
        2005: (APR, 22),
        2012: (FEB, 5),
        2014: (JAN, 14),
        2015: ((JAN, 3), (DEC, 24)),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2021: (OCT, 20),
        2023: (SEP, 28),
        2024: (SEP, 16),
        2025: (SEP, 5),
    }


class IndonesiaStaticHolidays:
    """Indonesia special holidays.

    References (Election Days):
        * <https://web.archive.org/web/20250413192438/https://peraturan.bpk.go.id/Details/58370/keppres-no-53-tahun-1999>
        * <https://web.archive.org/web/20250413192447/https://peraturan.bpk.go.id/Details/55580/keppres-no-26-tahun-2004>
        * <https://web.archive.org/web/20250413192644/https://peraturan.bpk.go.id/Details/55662/keppres-no-55-tahun-2004>
        * <https://web.archive.org/web/20250413192655/https://peraturan.bpk.go.id/Details/55763/keppres-no-75-tahun-2004>
        * <https://web.archive.org/web/20250413192523/https://peraturan.bpk.go.id/Details/55345/keppres-no-7-tahun-2009>
        * <https://web.archive.org/web/20250413192539/https://peraturan.bpk.go.id/Details/55369/keppres-no-17-tahun-2009>
        * <https://web.archive.org/web/20250413192619/https://peraturan.bpk.go.id/Details/57563/keppres-no-14-tahun-2014>
        * <https://web.archive.org/web/20250413192603/https://peraturan.bpk.go.id/Details/57581/keppres-no-24-tahun-2014>
        * <https://web.archive.org/web/20250413192707/https://peraturan.bpk.go.id/Details/54477/keppres-no-25-tahun-2015>
        * <https://web.archive.org/web/20250413192641/https://peraturan.bpk.go.id/Details/57397/keppres-no-3-tahun-2017>
        * <https://web.archive.org/web/20250413192642/https://peraturan.bpk.go.id/Details/82669/keppres-no-15-tahun-2018>
        * <https://web.archive.org/web/20250413192708/https://peraturan.bpk.go.id/Details/104497/keppres-no-10-tahun-2019>
        * <https://web.archive.org/web/20241226224121/https://peraturan.bpk.go.id/Details/152704/keppres-no-22-tahun-2020>
        * <https://web.archive.org/web/20250413192801/https://peraturan.bpk.go.id/Details/277337/keppres-no-10-tahun-2024>
        * <https://web.archive.org/web/20241104090706/https://www.buduk.desa.id/berita/read/83/surat-edaran-menteri-ketenagakerjaan-republik-indonesia-nomor-1-tahun-2024.html>

    References (Joint Holidays):
        * <https://web.archive.org/web/20240626171556/https://data.santoslolowang.com/viewing/Agama_Nasional_Indonesia/kepbersama_2002.pdf/>
        * <https://web.archive.org/web/20250413192812/https://nasional.tempo.co/read/5515/tahun-depan-libur-hari-raya-berubah>
        * <https://web.archive.org/web/20250413192839/https://nasional.tempo.co/read/45224/2005-pemerintah-tetapkan-13-hari-libur-nasional>
        * <https://web.archive.org/web/20250413192830/https://news.detik.com/berita/d-562326/30-maret-libur-hari-nyepi-31-maret-ditetapkan-cuti-bersama>
        * <https://web.archive.org/web/20250413192836/https://news.detik.com/berita/d-836627/dari-12-hari-jatah-cuti-tahun-ini-dipaksa-cuti-bareng-11-hari>
        * <https://web.archive.org/web/20250413192922/https://news.detik.com/berita/d-787189/inilah-jadwal-libur-dan-cuti-bersama-2008>
        * <https://web.archive.org/web/20250413192845/https://news.detik.com/berita/d-889403/cuti-bersama-dihapus-sisa-4-hari-untuk-lebaran-natal>
        * <https://web.archive.org/web/20250413193013/https://news.detik.com/berita/d-1263142/kamis-24-desember-cuti-bersama>
        * <https://web.archive.org/web/20250413192957/https://news.detik.com/berita/d-1179419/daftar-libur-dan-cuti-bersama-2010>
        * <https://web.archive.org/web/20250413192919/https://news.detik.com/berita/d-1378563/daftar-hari-libur-nasional-dan-cuti-bersama-2011>
        * <https://web.archive.org/web/20250413192931/https://news.detik.com/berita/d-1639209/pemerintah-tetapkan-senin-16-mei-2011-cuti-bersama>
        * <https://web.archive.org/web/20250413193003/https://news.detik.com/berita/d-1831855/cuti-bersama-tahun-2012-bertambah-1-hari-jadi-6-hari>
        * <https://web.archive.org/web/20250413192956/https://news.detik.com/berita/d-1969257/ini-dia-jadwal-hari-libur-nasional-cuti-bersama-tahun-2013>
        * <https://web.archive.org/web/20250413193006/https://news.detik.com/berita/d-2335984/daftar-cuti-bersama-dan-hari-libur-nasional-2014-termasuk-may-day>
        * <https://web.archive.org/web/20250413193133/https://news.detik.com/berita/d-2576131/ini-daftar-hari-libur-dan-cuti-bersama-tahun-2015>
        * <https://web.archive.org/web/20250413193040/https://bkpsdm.salatiga.go.id/pelaksanaan-hari-libur-nasional-dan-cuti-bersama-tahun-2015.html>
        * <https://web.archive.org/web/20250413193101/https://news.detik.com/berita/d-2952083/ini-rincian-libur-dan-cuti-bersama-2016>
        * <https://web.archive.org/web/20250413193107/https://news.detik.com/berita/d-3366218/libur-nasional-dan-cuti-bersama-2017-tambah-3-hari-ini-daftar-lengkapnya>
        * <https://web.archive.org/web/20250413193106/https://setkab.go.id/tidak-kurangi-hak-cuti-tahunan-presiden-jokowi-tetapkan-23-juni-sebagai-cuti-bersama-idul-fitri/>
        * <https://web.archive.org/web/20250413193200/https://setkab.go.id/pelayanan-publik-tetap-jalan-presiden-jokowi-tetapkan-cuti-bersama-pns-tahun-2018/>
        * <https://web.archive.org/web/20250413193130/https://setkab.go.id/keppres-no-132019-cuti-bersama-pns-pada-idul-fitri-1440h-tanggal-3-4-dan-7-juni/>
        * <https://web.archive.org/web/20250413193142/https://setkab.go.id/pemerintah-keluarkan-skb-3-menteri-hapus-tiga-hari-cuti-bersama-2020/>
        * <https://web.archive.org/web/20250413193249/https://setkab.go.id/inilah-perubahan-hari-libur-nasional-dan-cuti-bersama-tahun-2021/>
        * <https://web.archive.org/web/20250413193216/https://setkab.go.id/presiden-jokowi-terbitkan-keppres-4-tahun-2022-tentang-cuti-bersama-asn-tahun-2022/>
        * <https://web.archive.org/web/20250413193308/https://news.detik.com/berita/d-6468002/apakah-ada-cuti-bersama-natal-2022-cek-infonya-di-sini>
        * <https://web.archive.org/web/20250413193243/https://setkab.go.id/presiden-tandatangani-keppres-perubahan-cuti-bersama-bagi-asn/>
        * <https://web.archive.org/web/20250413193243/https://setkab.go.id/inilah-keppres-7-2024-tentang-cuti-bersama-asn-tahun-2024/>
        * <https://web.archive.org/web/20250413193349/https://setkab.go.id/pemerintah-tetapkan-hari-libur-nasional-dan-cuti-bersama-tahun-2025/>
        * <https://web.archive.org/web/20250802143125/https://news.detik.com/berita/d-8039857/pemerintah-tetapkan-senin-18-agustus-hari-libur>
    """

    # General Election Day.
    general_election_day = tr("Hari Pemilihan Unum")
    # Presidential Election Day.
    presidential_election_day = tr("Hari Pemilihan Presiden")
    # Legislative Election Day.
    legislative_election_day = tr("Hari Pemilihan Legislatif")
    # Local Election Day.
    local_election_day = tr("Hari Pemilihan Kepala Daerah")

    # Eid al-Fitr Joint Holiday.
    eid_al_fitr_joint_holiday = tr("Cuti Bersama Hari Raya Idul Fitri")
    # Eid al-Adha Joint Holiday.
    eid_al_adha_joint_holiday = tr("Cuti Bersama Hari Raya Idul Adha")
    # Prophet's Birthday Joint Holiday.
    mawlid_joint_holiday = tr("Cuti Bersama Maulid Nabi Muhammad")
    # Islamic New Year Joint Holiday.
    islamic_new_year_joint_holiday = tr("Cuti Bersama Tahun Baru Islam")
    # Ascension Joint Holiday.
    ascension_joint_holiday = tr("Cuti Bersama Kenaikan Yesus Kristus")
    # Christmas Joint Holiday.
    christmas_joint_holiday = tr("Cuti Bersama Hari Raya Natal")
    # Lunar New Year Joint Holiday.
    lunar_new_year_joint_holiday = tr("Cuti Bersama Tahun Baru Imlek")
    # Day of Silence Joint Holiday.
    day_of_silence_joint_holiday = tr("Cuti Bersama Hari Suci Nyepi")
    # Vesak Joint Holiday.
    vesak_joint_holiday = tr("Cuti Bersama Hari Raya Waisak")
    # New Year's Joint Holiday.
    new_years_joint_holiday = tr("Cuti Bersama Tahun Baru Masehi")
    # Independence Day Joint Holiday.
    independence_day_joint_holiday = tr("Cuti Bersama Hari Kemerdekaan Republik Indonesia")

    special_public_holidays = {
        1999: (JUN, 7, legislative_election_day),
        2004: (
            (APR, 5, legislative_election_day),
            (JUL, 5, presidential_election_day),
            (SEP, 20, presidential_election_day),
        ),
        2009: (
            (APR, 9, legislative_election_day),
            (JUL, 8, presidential_election_day),
        ),
        2014: (
            (APR, 9, legislative_election_day),
            (JUL, 9, presidential_election_day),
        ),
        2015: (DEC, 9, local_election_day),
        2017: (FEB, 15, local_election_day),
        2018: (JUN, 27, local_election_day),
        2019: (APR, 17, general_election_day),
        2020: (DEC, 9, local_election_day),
        2024: (
            (FEB, 14, general_election_day),
            (NOV, 27, local_election_day),
        ),
    }
    special_public_holidays_observed = {
        # Eid al-Fitr.
        2004: (NOV, 16, tr("Hari Raya Idul Fitri")),
    }
    special_government_holidays = {
        # Cuti Bersama (Joint Holidays/Collective Leaves).
        # This was first implemented in 2002.
        2002: (
            (DEC, 5, eid_al_fitr_joint_holiday),
            (DEC, 9, eid_al_fitr_joint_holiday),
            (DEC, 10, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2003: (
            (NOV, 24, eid_al_fitr_joint_holiday),
            (NOV, 27, eid_al_fitr_joint_holiday),
            (NOV, 28, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2004: (
            (NOV, 17, eid_al_fitr_joint_holiday),
            (NOV, 18, eid_al_fitr_joint_holiday),
            (NOV, 19, eid_al_fitr_joint_holiday),
        ),
        2005: (
            (NOV, 2, eid_al_fitr_joint_holiday),
            (NOV, 5, eid_al_fitr_joint_holiday),
            (NOV, 7, eid_al_fitr_joint_holiday),
            (NOV, 8, eid_al_fitr_joint_holiday),
        ),
        2006: (
            (MAR, 31, day_of_silence_joint_holiday),
            (MAY, 26, ascension_joint_holiday),
            (AUG, 18, independence_day_joint_holiday),
            (OCT, 23, eid_al_fitr_joint_holiday),
            (OCT, 26, eid_al_fitr_joint_holiday),
            (OCT, 27, eid_al_fitr_joint_holiday),
        ),
        2007: (
            (MAY, 18, ascension_joint_holiday),
            (OCT, 12, eid_al_fitr_joint_holiday),
            (OCT, 15, eid_al_fitr_joint_holiday),
            (OCT, 16, eid_al_fitr_joint_holiday),
            (OCT, 17, eid_al_fitr_joint_holiday),
            (OCT, 18, eid_al_fitr_joint_holiday),
            (OCT, 19, eid_al_fitr_joint_holiday),
            (DEC, 21, christmas_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
            (DEC, 31, new_years_joint_holiday),
        ),
        2008: (
            (JAN, 11, islamic_new_year_joint_holiday),
            (SEP, 29, eid_al_fitr_joint_holiday),
            (SEP, 30, eid_al_fitr_joint_holiday),
            (OCT, 3, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2009: (
            (JAN, 2, new_years_joint_holiday),
            (SEP, 18, eid_al_fitr_joint_holiday),
            (SEP, 23, eid_al_fitr_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
        ),
        2010: (
            (SEP, 9, eid_al_fitr_joint_holiday),
            (SEP, 13, eid_al_fitr_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
        ),
        2011: (
            (MAY, 16, vesak_joint_holiday),
            (AUG, 29, eid_al_fitr_joint_holiday),
            (SEP, 1, eid_al_fitr_joint_holiday),
            (SEP, 2, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2012: (
            (MAY, 18, ascension_joint_holiday),
            (AUG, 21, eid_al_fitr_joint_holiday),
            (AUG, 22, eid_al_fitr_joint_holiday),
            (NOV, 16, islamic_new_year_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
            (DEC, 31, new_years_joint_holiday),
        ),
        2013: (
            (AUG, 5, eid_al_fitr_joint_holiday),
            (AUG, 6, eid_al_fitr_joint_holiday),
            (AUG, 7, eid_al_fitr_joint_holiday),
            (OCT, 14, eid_al_adha_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2014: (
            (JUL, 30, eid_al_fitr_joint_holiday),
            (JUL, 31, eid_al_fitr_joint_holiday),
            (AUG, 1, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2015: (
            (JUL, 16, eid_al_fitr_joint_holiday),
            (JUL, 20, eid_al_fitr_joint_holiday),
            (JUL, 21, eid_al_fitr_joint_holiday),
        ),
        2016: (
            (JUL, 4, eid_al_fitr_joint_holiday),
            (JUL, 5, eid_al_fitr_joint_holiday),
            (JUL, 8, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2017: (
            (JAN, 2, new_years_joint_holiday),
            (JUN, 23, eid_al_fitr_joint_holiday),
            (JUN, 27, eid_al_fitr_joint_holiday),
            (JUN, 28, eid_al_fitr_joint_holiday),
            (JUN, 29, eid_al_fitr_joint_holiday),
            (JUN, 30, eid_al_fitr_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2018: (
            (JUN, 11, eid_al_fitr_joint_holiday),
            (JUN, 12, eid_al_fitr_joint_holiday),
            (JUN, 13, eid_al_fitr_joint_holiday),
            (JUN, 14, eid_al_fitr_joint_holiday),
            (JUN, 18, eid_al_fitr_joint_holiday),
            (JUN, 19, eid_al_fitr_joint_holiday),
            (JUN, 20, eid_al_fitr_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
        ),
        2019: (
            (JUN, 3, eid_al_fitr_joint_holiday),
            (JUN, 4, eid_al_fitr_joint_holiday),
            (JUN, 7, eid_al_fitr_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
        ),
        2020: (
            (AUG, 21, islamic_new_year_joint_holiday),
            (OCT, 28, mawlid_joint_holiday),
            (OCT, 30, mawlid_joint_holiday),
            (DEC, 24, christmas_joint_holiday),
        ),
        2021: (MAY, 12, eid_al_fitr_joint_holiday),
        2022: (
            (APR, 29, eid_al_fitr_joint_holiday),
            (MAY, 4, eid_al_fitr_joint_holiday),
            (MAY, 5, eid_al_fitr_joint_holiday),
            (MAY, 6, eid_al_fitr_joint_holiday),
        ),
        2023: (
            (JAN, 23, lunar_new_year_joint_holiday),
            (MAR, 23, day_of_silence_joint_holiday),
            (APR, 19, eid_al_fitr_joint_holiday),
            (APR, 20, eid_al_fitr_joint_holiday),
            (APR, 21, eid_al_fitr_joint_holiday),
            (APR, 24, eid_al_fitr_joint_holiday),
            (APR, 25, eid_al_fitr_joint_holiday),
            (JUN, 2, vesak_joint_holiday),
            (JUN, 28, eid_al_adha_joint_holiday),
            (JUN, 30, eid_al_adha_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2024: (
            (FEB, 9, lunar_new_year_joint_holiday),
            (MAR, 12, day_of_silence_joint_holiday),
            (APR, 8, eid_al_fitr_joint_holiday),
            (APR, 9, eid_al_fitr_joint_holiday),
            (APR, 12, eid_al_fitr_joint_holiday),
            (APR, 15, eid_al_fitr_joint_holiday),
            (MAY, 10, ascension_joint_holiday),
            (MAY, 24, vesak_joint_holiday),
            (JUN, 18, eid_al_adha_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
        2025: (
            (JAN, 28, lunar_new_year_joint_holiday),
            (MAR, 28, day_of_silence_joint_holiday),
            (APR, 2, eid_al_fitr_joint_holiday),
            (APR, 3, eid_al_fitr_joint_holiday),
            (APR, 4, eid_al_fitr_joint_holiday),
            (APR, 7, eid_al_fitr_joint_holiday),
            (MAY, 13, vesak_joint_holiday),
            (MAY, 30, ascension_joint_holiday),
            (JUN, 9, eid_al_adha_joint_holiday),
            (AUG, 18, independence_day_joint_holiday),
            (DEC, 26, christmas_joint_holiday),
        ),
    }
    special_government_holidays_observed = {
        2020: (DEC, 31, eid_al_fitr_joint_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from functools import cache

from holidays.calendars.gregorian import _timedelta

MO = 1954168.050623  # Beginning of 0 ME for MMT.
SY = 1577917828.0 / 4320000.0  # Solar year (365.2587565 days).


class _BurmeseLunisolar:
    """Burmese Lunisolar calendar.

    References:
        * [Algorithm, Program and Calculation of Myanmar Calendar](https://web.archive.org/web/20250510011425/http://cool-emerald.blogspot.com/2013/06/algorithm-program-and-calculation-of.html)
    """

    START_DATE = date(1939, 3, 20)  # 1 Late Tagu 1301 Burmese Era.
    START_YEAR = 1939  # 1301 Burmese Era.
    END_YEAR = 2100  # 1463 Burmese Era.

    LITTLE_WATAT_YEARS_GREGORIAN = {
        1939,
        1948,
        1955,
        1958,
        1966,
        1972,
        1974,
        1982,
        1988,
        1993,
        1999,
        2004,
        2012,
        2018,
        2020,
        2029,
        2034,
        2039,
        2045,
        2050,
        2056,
        2061,
        2069,
        2075,
        2077,
        2086,
        2091,
        2096,
    }

    BIG_WATAT_YEARS_GREGORIAN = {
        1942,
        1945,
        1950,
        1953,
        1961,
        1964,
        1969,
        1977,
        1980,
        1985,
        1991,
        1996,
        2001,
        2007,
        2010,
        2015,
        2023,
        2026,
        2031,
        2037,
        2042,
        2048,
        2053,
        2058,
        2064,
        2067,
        2072,
        2080,
        2083,
        2088,
        2094,
        2099,
    }

    @staticmethod
    def jdn_to_gregorian(jdn: int) -> date:
        """Convert Julian Day Number to Gregorian date.

        Args:
            jdn: Julian Day Number.

        Returns:
            Gregorian date.
        """
        j = jdn - 1721119
        y, j = divmod(4 * j - 1, 146097)
        d = j // 4
        j, d = divmod(4 * d + 3, 1461)
        d = (d + 4) // 4
        m, d = divmod(5 * d - 3, 153)
        d = (d + 5) // 5
        y = 100 * y + j
        if m < 10:
            m += 3
        else:
            m -= 9
            y += 1

        return date(y, m, d)

    @cache
    def _get_start_date(self, year: int) -> date | None:
        if year < self.START_YEAR or year > self.END_YEAR:
            return None

        delta_days = 354 * (year - self.START_YEAR)
        for iter_year in range(self.START_YEAR, year):
            if iter_year in self.LITTLE_WATAT_YEARS_GREGORIAN:
                delta_days += 30
            elif iter_year in self.BIG_WATAT_YEARS_GREGORIAN:
                delta_days += 31

        return _timedelta(self.START_DATE, delta_days)

    def thingyan_dates(self, year: int) -> tuple[date | None, date | None]:
        """Calculate key dates of Thingyan (Myanmar New Year festival) - Akya day
            and Atat day.

        Args:
            year:
                Gregorian year.

        Returns:
            Akya day date, Atat day date.

        """
        if year < self.START_YEAR or year > self.END_YEAR:
            return None, None

        ja = SY * (year - 638) + MO
        jk = (ja - 2.169918982) if year >= 1950 else (ja - 2.1675)

        return self.jdn_to_gregorian(round(jk)), self.jdn_to_gregorian(round(ja))

    def kason_full_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of Full Moon Day of Kason.

        15th day of 2nd month (Kason).

        To calculate, we use the following time delta:

        * All years:
            29 [1] + 15 - 1 = 43.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of Full Moon Day of Kason.
            `None` if the Gregorian year is out of supported range.
        """
        if not (start_date := self._get_start_date(year)):
            return None

        return _timedelta(start_date, +43)

    def waso_full_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of Full Moon Day of Waso.

        15th day of 4th month (Waso).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 266 [4-12] + 15 - 1 = Year_length - 252.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of Full Moon Day of Waso.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -252)

    def thadingyut_full_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of Full Moon Day of Thadingyut.

        15th day of 7th month (Thadingyut).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 177 [7-12] + 15 - 1 = Year_length - 163.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of Full Moon Day of Thadingyut.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -163)

    def tazaungmon_waxing_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of 1st Waxing Day of Tazaungmon.

        1st day of 8th month (Tazaungmon).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 148 [8-12] + 1 - 1 = Year_length - 148.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of 1st Waxing Day of Tazaungmon.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -148)

    def tazaungmon_full_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of Full Moon Day of Tazaungmon.

        15th day of 8th month (Tazaungmon).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 148 [8-12] + 15 - 1 = Year_length - 134.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of Full Moon Day of Tazaungmon.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -134)

    def pyatho_waxing_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of 1st Waxing Day of Pyatho.

        1st day of 10th month (Pyatho).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 89 [10-12] + 1 - 1 = Year_length - 89.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of 1st Waxing Day of Pyatho.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -89)

    def tabaung_full_moon_date(self, year: int) -> date | None:
        """Calculate the Gregorian date of Full Moon Day of Tabaung.

        15th day of 12th month (Tabaung).

        To calculate, we use the following time delta:

        * All years:
            Year_length - 30 [12] + 15 - 1 = Year_length - 16.

        Args:
            year:
                Gregorian year.

        Returns:
            Gregorian date of Full Moon Day of Tabaung.
            `None` if the Gregorian year is out of supported range.
        """
        if not (next_year_start_date := self._get_start_date(year + 1)):
            return None

        return _timedelta(next_year_start_date, -16)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import MAR, APR, JUN, JUL, AUG, SEP, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Guinea(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Guinea holidays.

    References:
        * [Decree No. 2022-0526](https://archive.org/details/d-2022-0526-prg-cnrd-221103-131021)
        * <https://web.archive.org/web/20250408180953/https://www.timeanddate.com/holidays/guinea/>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Guinea>
        * <https://web.archive.org/web/20241205013852/https://anydayguide.com/calendar/1878>
        * <https://web.archive.org/web/20250408180953/https://web.archive.org/web/20240622164935/https://www.timeanddate.com/holidays/guinea/second-republic-day>
        * <https://web.archive.org/web/20250408180953/https://web.archive.org/web/20240806075538/https://www.timeanddate.com/holidays/guinea/all-saints-day>

    According to Decree No. 2022-0526 of 2 November 2022:
        * Eid al-Adha became a two-day holiday (Article 1).
        * If New Year's Day, Independence Day or Eid al-Fitr fall on a non-working day,
            the next working day is also a holiday (Article 2).
    """

    country = "GN"
    default_language = "fr"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    # Day after the %s.
    observed_label = tr("Lendemain de la %s")
    # Day after the %s (estimated).
    observed_estimated_label = tr("Lendemain de la %s (estimé)")
    supported_languages = ("en_US", "fr")

    # Guinea gained independence from France on October 2, 1958.
    start_year = 1959

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=GuineaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        dt = self._add_new_years_day(tr("Fête du Nouvel an"))
        if self._year >= 2023:
            self._add_observed(dt)

        if 1985 <= self._year <= 2021:
            # Second Republic Day.
            self._add_holiday_apr_3(tr("Jour de la Deuxième République"))

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        # Labor Day.
        self._add_labor_day(tr("Fête du Travail"))

        # Africa Day.
        self._add_africa_day(tr("Anniversaire de l'Union Africaine"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assomption"))

        # Independence Day.
        dt = self._add_holiday_oct_2(tr("Fête anniversaire de l'indépendance de la Guinée"))
        if self._year >= 2023:
            self._add_observed(dt)

        if self._year <= 2021:
            # All Saints' Day.
            self._add_all_saints_day(tr("Toussaint"))

        # Christmas Day.
        self._add_christmas_day(tr("Fête de Noël"))

        # Day after Prophet's Birthday.
        self._add_mawlid_day(tr("Lendemain de la nuit du Maoloud"))

        # Day after Night of Power.
        self._add_laylat_al_qadr_day(tr("Lendemain de la nuit Lailatoul Qadr"))

        # Eid al-Fitr.
        for dt in self._add_eid_al_fitr_day(tr("Jour de l'Aïd el-Fitr")):
            if self._year >= 2023:
                self._add_observed(dt)

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Jour de la Tabaski"))

        if self._year >= 2023:
            # Day after Eid al-Adha.
            self._add_eid_al_adha_day_two(tr("Lendemain de la Tabaski"))


class GN(Guinea):
    pass


class GIN(Guinea):
    pass


class GuineaIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2015, 2025)
    EID_AL_ADHA_DATES = {
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
        2022: (JUL, 10),
        2025: (JUN, 7),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2015, 2025)
    EID_AL_FITR_DATES = {
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2025: (MAR, 31),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2015, 2025)
    MAWLID_DATES = {
        2015: (DEC, 24),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2019: (NOV, 10),
        2025: (SEP, 5),
    }

    LAYLAT_AL_QADR_DATES_CONFIRMED_YEARS = (2015, 2025)
    LAYLAT_AL_QADR_DATES = {
        2016: (JUL, 3),
        2018: (JUN, 12),
        2022: (APR, 29),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays import APR, MAY, JUN, JUL, SEP, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class CaymanIslands(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Cayman Islands holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Cayman_Islands>
        * [Public Holidays Law (2007 Revision)](https://web.archive.org/web/20250227060525/https://legislation.gov.ky/cms/images/LEGISLATION/PRINCIPAL/1964/1964-0140/PublicHolidaysAct_2007%20Revision_g.pdf)
        * [Public Holidays Order, 2024](https://web.archive.org/web/20240518181823/https://legislation.gov.ky/cms/images/LEGISLATION/AMENDING/2024/2024-O004/PublicHolidaysOrder2024SL4of2024.pdf)
        * [2006-2015](https://web.archive.org/web/20151014061601/http://www.gov.ky/portal/page?_pageid=1142,1592653&_dad=portal&_schema=PORTAL)
        * [2016-2018](https://web.archive.org/web/20180330170202/http://www.gov.ky:80/portal/page/portal/cighome/cayman/islands/publicholidays)
        * [2021](http://archive.today/2025.07.09-033240/https://www.gov.ky/news/press-release-details/public-holidays-for-2021)
        * [2022](http://archive.today/2025.07.09-033515/https://www.gov.ky/news/press-release-details/public-holidays-2022)
        * [2024](http://archive.today/2025.01.06-110234/https://www.gov.ky/calendar/public-holidays)
        * [2025](http://archive.today/2025.07.09-033853/https://www.gov.ky/calendar/public-holidays)
    """

    country = "KY"
    default_language = "en_GB"
    # %s observed.
    observed_label = tr("%s (observed)")
    # Earliest year of holidays with an accessible online record.
    start_year = 2006
    supported_languages = ("en_GB", "en_US")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=CaymanIslandsStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # National Heroes Day.
        self._add_holiday_4th_mon_of_jan(tr("National Heroes Day"))

        # Ash Wednesday.
        self._add_ash_wednesday(tr("Ash Wednesday"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        if self._year >= 2024:
            # Emancipation Day.
            self._add_holiday_1st_mon_of_may(tr("Emancipation Day"))

        # Discovery Day.
        self._add_holiday_3rd_mon_of_may(tr("Discovery Day"))

        if self._year <= 2022:
            queens_birthday_dates = {
                2007: (JUN, 18),
                2012: (JUN, 18),
                2013: (JUN, 17),
                2017: (JUN, 19),
                2022: (JUN, 6),
            }
            # Queen's Birthday.
            name = tr("Queen's Birthday")
            if dt := queens_birthday_dates.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_holiday_2_days_past_2nd_sat_of_jun(name)
        else:
            # King's Birthday.
            self._add_holiday_2_days_past_3rd_sat_of_jun(tr("King's Birthday"))

        # Constitution Day.
        self._add_holiday_1st_mon_of_jul(tr("Constitution Day"))

        # Remembrance Day.
        self._add_holiday_2nd_mon_of_nov(tr("Remembrance Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class KY(CaymanIslands):
    pass


class CYM(CaymanIslands):
    pass


class CaymanIslandsStaticHolidays:
    """Cayman Islands special holidays.

    References:
        * [Public Holidays Order, 2009](https://archive.org/details/public-holidays-order-2009-sl-33-of-2009)
        * [Public Holidays Order (No. 2), 2019](https://archive.org/details/public-holidays-no.-2-order-2019-sl-38-of-2019)
        * [Public Holidays Order, 2025](https://archive.org/details/public-holidays-order-2025-sl-15-of-2025)
        * [Referendum Day 2019](https://web.archive.org/web/20250711121821/https://www.facebook.com/ElectionsOffice/posts/reminder-the-referendum-vote-has-been-postponed-and-will-no-longer-take-place-th/2619995141431734/?_rdc=2&_rdr)
        * [UK Royal Wedding](https://en.wikipedia.org/wiki/Wedding_of_Prince_William_and_Catherine_Middleton)
        * [Queen Elizabeth II's Diamond Jubilee](https://web.archive.org/web/20210803202236/https://www.caymancompass.com/2012/06/06/queens-diamond-jubilee-feted/)
        * [Queen Elizabeth II's Funeral](https://web.archive.org/web/20231226055510/https://www.caymancompass.com/2022/09/12/cayman-declares-public-holiday-for-queens-funeral/)
        * [King Charles III's Coronation](https://web.archive.org/web/20250601214328/https://www.radiocayman.gov.ky/news/public-holidays-for-2023-unconfirmed)
    """

    # Referendum Day.
    referendum_day_name = tr("Referendum Day")
    # General Election Day.
    general_election_day_name = tr("General Election Day")
    special_public_holidays = {
        2009: (
            # 2009 Cayman Islands Constitution Day.
            (NOV, 6, tr("2009 Cayman Islands Constitution Day")),
            (MAY, 20, general_election_day_name),
        ),
        # UK Royal Wedding.
        2011: (APR, 29, tr("UK Royal Wedding")),
        2012: (
            # Queen Elizabeth II's Diamond Jubilee.
            (JUN, 4, tr("Queen Elizabeth II's Diamond Jubilee")),
            (JUL, 18, referendum_day_name),
        ),
        2013: (MAY, 22, general_election_day_name),
        2017: (MAY, 24, general_election_day_name),
        2019: (DEC, 19, referendum_day_name),
        2021: (APR, 14, general_election_day_name),
        # Queen Elizabeth II's Funeral.
        2022: (SEP, 19, tr("Queen Elizabeth II's Funeral")),
        # King Charles III's Coronation.
        2023: (MAY, 8, tr("Coronation of His Majesty King Charles III")),
        2025: (APR, 30, general_election_day_name),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import (
    FEB,
    MAR,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    THU,
    FRI,
    SAT,
    _timedelta,
)
from holidays.groups import IslamicHolidays
from holidays.holiday_base import HolidayBase


class Oman(HolidayBase, IslamicHolidays):
    """Oman holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Oman>
        * [Independence](https://web.archive.org/web/20250211055127/https://omaninfo.om/pages/175/show/572)
        * [Weekend](https://web.archive.org/web/20250213121638/https://abnnews.com/the-sultanate-of-oman-changes-weekend-days-from-01-may-2013/)
        * [Decree 56/2020](https://web.archive.org/web/20240522150742/https://decree.om/2020/rd20200056/)
        * [Decree 88/2022](https://web.archive.org/web/20221207194900/https://decree.om/2022/rd20220088/)
        * [Decree 15/2025 (National day is moved)](https://web.archive.org/web/20250121132446/https://decree.om/2025/rd20250015/)
    """

    country = "OM"
    default_language = "ar"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    start_year = 1970
    supported_languages = ("ar", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        IslamicHolidays.__init__(
            self, cls=OmanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _get_weekend(self, dt: date) -> set[int]:
        # Oman switches from THU-FRI to FRI-SAT on May 1, 2013.
        return {FRI, SAT} if dt >= date(2013, MAY, 1) else {THU, FRI}

    def _populate_public_holidays(self):
        if self._year >= 2020:
            # Sultan's Accession Day.
            self._add_holiday_jan_11(tr("اليوم الوطني لتولي السلطان"))

        if self._year <= 2019:
            # Renaissance Day.
            self._add_holiday_jul_23(tr("يوم النهضة"))

        if self._year >= 2020:
            # National Day.
            name = tr("يوم وطني")
            if self._year <= 2024:
                self._add_holiday_nov_18(name)
                self._add_holiday_nov_19(name)
            else:
                self._add_holiday_nov_20(name)
                self._add_holiday_nov_21(name)

        # Islamic New Year.
        self._add_islamic_new_year_day(tr("رأس السنة الهجرية"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("مولد النبي"))

        # Isra' and Mi'raj.
        self._add_isra_and_miraj_day(tr("الإسراء والمعراج"))

        # Eid al-Fitr.
        name = tr("عيد الفطر")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)
        for dt in self._add_holiday_29_ramadan(name):
            if _timedelta(dt, +1) not in self:
                self._add_eid_al_fitr_eve(name)

        # Eid al-Adha.
        name = tr("عيد الأضحى")
        self._add_arafah_day(name)
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)


class OM(Oman):
    pass


class OMN(Oman):
    pass


class OmanIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20240911084650/https://www.timeanddate.com/holidays/oman/muharram-new-year
    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2018, 2024)
    HIJRI_NEW_YEAR_DATES = {
        2019: (SEP, 1),
        2020: (AUG, 21),
        2021: (AUG, 10),
        2023: (JUL, 20),
    }

    # https://web.archive.org/web/20240810230306/https://www.timeanddate.com/holidays/oman/prophet-birthday
    MAWLID_DATES_CONFIRMED_YEARS = (2018, 2024)
    MAWLID_DATES = {
        2021: (OCT, 19),
        2022: (OCT, 9),
        2023: (SEP, 28),
    }

    # https://web.archive.org/web/20241231191036/https://www.timeanddate.com/holidays/oman/isra-miraj
    ISRA_AND_MIRAJ_DATES_CONFIRMED_YEARS = (2018, 2025)
    ISRA_AND_MIRAJ_DATES = {
        2022: (MAR, 1),
        2023: (FEB, 19),
    }

    # https://web.archive.org/web/20240913230603/https://www.timeanddate.com/holidays/oman/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2019, 2024)
    EID_AL_ADHA_DATES = {
        2024: (JUN, 17),
    }

    # https://web.archive.org/web/20241231191036/https://www.timeanddate.com/holidays/oman/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2019, 2025)
    EID_AL_FITR_DATES = {
        2023: (APR, 22),
        2025: (MAR, 31),
    }

    # https://web.archive.org/web/20240814104839/https://www.timeanddate.com/holidays/oman/ramadan-begins
    RAMADAN_BEGINNING_DATES_CONFIRMED_YEARS = (2023, 2025)
    RAMADAN_BEGINNING_DATES = {
        2024: (MAR, 12),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Croatia(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Croatia holidays.

    References:
        * <https://web.archive.org/web/20250206084954/https://narodne-novine.nn.hr/clanci/sluzbeni/2019_11_110_2212.html>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Croatia>
        * <https://hr.wikipedia.org/wiki/Blagdani_i_spomendani_u_Hrvatskoj>

    Updated with act 022-03 / 19-01 / 219 of 14 November 2019
    """

    country = "HR"
    default_language = "hr"
    supported_languages = ("en_US", "hr", "uk")
    start_year = 1992

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nova godina"))

        if self._year != 2002:
            # Epiphany.
            self._add_epiphany_day(tr("Bogojavljenje ili Sveta tri kralja"))

        if self._year >= 2009:
            # Easter Sunday.
            self._add_easter_sunday(tr("Uskrs"))

        # Easter Monday.
        self._add_easter_monday(tr("Uskrsni ponedjeljak"))

        if self._year >= 2002:
            # Corpus Christi.
            self._add_corpus_christi_day(tr("Tijelovo"))

        # Labor Day.
        self._add_labor_day(tr("Praznik rada"))

        if self._year >= 1996:
            # Statehood Day.
            name = tr("Dan državnosti")
            if 2002 <= self._year <= 2019:
                self._add_holiday_jun_25(name)
            else:
                self._add_holiday_may_30(name)

        # Anti-Fascist Struggle Day.
        self._add_holiday_jun_22(tr("Dan antifašističke borbe"))

        self._add_holiday_aug_5(
            # Victory and Homeland Thanksgiving Day and Croatian Veterans Day.
            tr("Dan pobjede i domovinske zahvalnosti i Dan hrvatskih branitelja")
            if self._year >= 2008
            # Victory and Homeland Thanksgiving Day.
            else tr("Dan pobjede i domovinske zahvalnosti")
        )

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Velika Gospa"))

        if 2002 <= self._year <= 2019:
            # Independence Day.
            self._add_holiday_oct_8(tr("Dan neovisnosti"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Svi sveti"))

        if self._year >= 2020:
            self._add_holiday_nov_18(
                # Remembrance Day.
                tr(
                    "Dan sjećanja na žrtve Domovinskog rata i "
                    "Dan sjećanja na žrtvu Vukovara i Škabrnje"
                )
            )

        # Christmas Day.
        self._add_christmas_day(tr("Božić"))

        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Sveti Stjepan"))


class HR(Croatia):
    pass


class HRV(Croatia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Nauru(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Nauru holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Nauru>
        * [Public Service Act 2016](https://web.archive.org/web/20250613095858/https://ronlaw.gov.nr/assets/docs/acts/2016/Public%20Service%20Act%202016_serv4.pdf)
        * [Nauru's Online Legal Database](https://web.archive.org/web/20250501021029/https://ronlaw.gov.nr/)
        * <https://web.archive.org/web/20250408012244/https://www.timeanddate.com/holidays/nauru/>

    Gazettes:
        * [Public Holidays 2019](https://web.archive.org/web/20250613095031/https://ronlaw.gov.nr/assets/docs/gazettes/2018/11/Gazette%20No.%20175%20(28%20November%202018).pdf)
        * [Public Holidays 2021](https://web.archive.org/web/20250613102242/https://ronlaw.gov.nr/assets/docs/gazettes/2020/12/Gazette%20No.%20227%20(21%20December%202020).pdf)
        * [Public Holidays 2022](https://web.archive.org/web/20250613102128/https://ronlaw.gov.nr/assets/docs/gazettes/2022/01/Gazette%20No.%207%20(6%20January%202022).pdf)
        * [Public Holidays 2023](https://web.archive.org/web/20250613101855/https://ronlaw.gov.nr/assets/docs/gazettes/2022/12/Gazette%20No.%20330%20(30%20December%202022).pdf)
        * [Public Holidays 2024](https://web.archive.org/web/20250613094703/https://ronlaw.gov.nr/assets/docs/gazettes/2024/01/Nauru%20Government%20Gazette,%20No.%2014%20(15%20January%202024).pdf)
        * [RONPHOS Handover Day 2018](https://web.archive.org/web/20250613095226/https://ronlaw.gov.nr/assets/docs/gazettes/2018/06/Gazette%20No.%20102%20(29%20June%202018).pdf)
        * [Ibumin Earoeni Day 2019](https://web.archive.org/web/20250613095413/https://ronlaw.gov.nr/assets/docs/gazettes/2019/08/Gazette%20No.%20139%20(12%20August%202019).pdf)
        * [Sir Hammer DeRoburt Day 2020](https://web.archive.org/web/20250614102647/https://ronlaw.gov.nr/assets/docs/gazettes/2020/09/Gazette%20No.%20171%20(10%20September%202020).pdf)
    """

    country = "NR"
    default_language = "en_NR"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_NR", "en_US")
    # Nauru gained independence on January 31, 1968.
    start_year = 1969

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        self._add_observed(
            # Independence Day.
            self._add_holiday_jan_31(tr("Independence Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Day following Independence Day.
            self._add_holiday_feb_1(tr("Day following Independence Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        if self._year >= 2019:
            # International Women's Day.
            self._add_observed(self._add_womens_day(tr("International Women's Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Easter Tuesday.
        self._add_easter_tuesday(tr("Easter Tuesday"))

        # Constitution Day.
        self._add_observed(self._add_holiday_may_17(tr("Constitution Day")))

        if self._year >= 2018:
            # RONPHOS Handover.
            self._add_observed(self._add_holiday_jul_1(tr("RONPHOS Handover")))

        if self._year >= 2019:
            # Ibumin Earoeni Day.
            self._add_observed(self._add_holiday_aug_19(tr("Ibumin Earoeni Day")))

        if self._year >= 2001:
            self._add_observed(
                self._add_holiday_sep_25(
                    # Sir Hammer DeRoburt Day.
                    tr("Sir Hammer DeRoburt Day")
                    if self._year >= 2020
                    # National Youth Day.
                    else tr("National Youth Day")
                )
            )

        # Angam Day.
        self._add_observed(self._add_holiday_oct_26(tr("Angam Day")))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Day following Christmas.
            self._add_christmas_day_two(tr("Day following Christmas")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class NR(Nauru):
    pass


class NRU(Nauru):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import MAR, JUL, AUG, SEP
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Zambia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Zambia holidays.

    References:
        * <https://web.archive.org/web/20241206135307/https://www.officeholidays.com/countries/zambia>
        * <https://web.archive.org/web/20250408170158/https://www.timeanddate.com/holidays/zambia/>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Zambia>
        * [The Public Holidays Act (1987)](https://web.archive.org/web/20231208015918/https://www.parliament.gov.zm/sites/default/files/documents/acts/Public%20Holidays%20Act.pdf)
    """

    country = "ZM"
    observed_label = "%s (observed)"
    start_year = 1965

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, ZambiaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        if self._year >= 1991:
            self._add_observed(
                # International Women's Day.
                self._add_womens_day("International Women's Day")
            )

        # Youth Day.
        self._add_observed(self._add_holiday_mar_12("Youth Day"))

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Holy Saturday.
        self._add_holy_saturday("Holy Saturday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        if self._year >= 2022:
            # Kenneth Kaunda Day.
            self._add_observed(self._add_holiday_apr_28("Kenneth Kaunda Day"))

        # Labour Day.
        self._add_observed(self._add_labor_day("Labour Day"))

        # Africa Freedom Day.
        self._add_observed(self._add_africa_day("Africa Freedom Day"))

        # Heroes' Day.
        self._add_holiday_1st_mon_of_jul("Heroes' Day")

        # Unity Day.
        self._add_holiday_1_day_past_1st_mon_of_jul("Unity Day")

        # Farmers' Day.
        self._add_holiday_1st_mon_of_aug("Farmers' Day")

        if self._year >= 2015:
            # National Prayer Day.
            self._add_observed(self._add_holiday_oct_18("National Prayer Day"))

        # Independence Day.
        self._add_observed(self._add_holiday_oct_24("Independence Day"))

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"))


class ZM(Zambia):
    pass


class ZMB(Zambia):
    pass


class ZambiaStaticHolidays:
    special_public_holidays = {
        2016: (
            (AUG, 11, "General elections and referendum"),
            (SEP, 13, "Inauguration ceremony of President-elect and Vice President-elect"),
        ),
        2018: (
            (MAR, 9, "Public holiday"),
            (JUL, 26, "Lusaka mayoral and other local government elections"),
        ),
        2021: (
            (JUL, 2, "Memorial service for Kenneth Kaunda"),
            (JUL, 7, "Funeral of Kenneth Kaunda"),
            (AUG, 12, "General elections"),
            (AUG, 13, "Counting in general elections"),
            (AUG, 24, "Presidential inauguration"),
        ),
        2022: (MAR, 18, "Funeral of Rupiah Banda"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import GOVERNMENT, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Andorra(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Andorra holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Andorra>
        * [Andorra's Constitution](https://web.archive.org/web/20250506145838/https://www.bopa.ad/Legislacio/Detall?doc=7586)
        * [Government holidays source](https://web.archive.org/web/20241005040546/https://seu.consellgeneral.ad/calendariPublic/show)
        * [2025](https://web.archive.org/web/20250506142149/https://www.bopa.ad/Documents/Detall?doc=GD_2024_10_24_14_36_16)

    Subdivisions holidays:
        * Canillo:
            * [2020](https://web.archive.org/web/20250506143618/https://www.bopa.ad/Documents/Detall?doc=QCH20191209_09_01_14)
            * [2021](https://web.archive.org/web/20250506143707/https://www.bopa.ad/Documents/Detall?doc=QCH20201130_11_52_35)
            * [2022](https://web.archive.org/web/20250506143404/https://www.bopa.ad/Documents/Detall?doc=QCD20211126_12_16_57)
            * [2023](https://web.archive.org/web/20250506143725/https://www.bopa.ad/Documents/Detall?doc=QCH20221010_11_26_39)
            * [2024](https://web.archive.org/web/20250506143740/https://www.bopa.ad/Documents/Detall?doc=QCH_2023_12_06_16_16_52)
            * [2025](https://web.archive.org/web/20250506144017/https://www.bopa.ad/Documents/Detall?doc=QCH_2024_10_10_11_42_15)
        * Encamp:
            * [2024](https://web.archive.org/web/20241201165207/https://www.bopa.ad/Documents/Detall?doc=QEH_2023_12_15_13_03_32)
            * [2025](https://web.archive.org/web/20241212121414/https://www.bopa.ad/Documents/Detall?doc=QEH_2024_11_27_10_29_49)
        * La Massana:
            * [2020](https://web.archive.org/web/20250506144157/https://www.bopa.ad/Documents/Detall?doc=QMH20191209_09_08_10)
            * [2021](https://web.archive.org/web/20250506144241/https://www.bopa.ad/Documents/Detall?doc=QMH20201218_11_52_59)
            * [2022](https://web.archive.org/web/20250506144526/https://www.bopa.ad/Documents/Detall?doc=QMH20211116_12_23_19)
            * [2023](https://web.archive.org/web/20250506144455/https://www.bopa.ad/Documents/Detall?doc=QMH20221216_12_41_13)
            * [2024](https://web.archive.org/web/20250506144500/https://www.bopa.ad/Documents/Detall?doc=QMH_2023_12_11_10_54_27)
            * [2025](https://web.archive.org/web/20250506144645/https://www.bopa.ad/Documents/Detall?doc=QMH_2024_11_19_09_10_48)
        * Ordino:
            * [2020](https://web.archive.org/web/20250506144555/https://www.bopa.ad/Documents/Detall?doc=QOH20191127_11_43_45)
            * [2021](https://web.archive.org/web/20250506144713/https://www.bopa.ad/Documents/Detall?doc=QOH20201229_12_24_42)
            * [2022](https://web.archive.org/web/20250506144715/https://www.bopa.ad/Documents/Detall?doc=QOH20211125_15_57_33)
            * [2023](https://web.archive.org/web/20250506144738/https://www.bopa.ad/Documents/Detall?doc=QOH20221228_10_02_52)
            * [2024](https://web.archive.org/web/20250506144729/https://www.bopa.ad/Documents/Detall?doc=QOH_2023_12_28_09_52_56)
            * [2025](https://web.archive.org/web/20250506144815/https://www.bopa.ad/Documents/Detall?doc=QOH_2024_11_28_13_42_34)
        * Sant Julià de Lòria:
            * [2020](https://web.archive.org/web/20250506144715/https://www.bopa.ad/Documents/Detall?doc=QSH20191122_11_45_30)
            * [2021](https://web.archive.org/web/20250506144827/https://www.bopa.ad/Documents/Detall?doc=QSH20201223_14_47_40)
            * [2022](https://web.archive.org/web/20250506144843/https://www.bopa.ad/Documents/Detall?doc=QSH20211217_08_48_18)
            * [2023](https://web.archive.org/web/20250506144859/https://www.bopa.ad/Documents/Detall?doc=QSH20221216_10_26_20)
            * [2024](https://web.archive.org/web/20250506145411/https://www.bopa.ad/Documents/Detall?doc=QSH_2023_12_07_11_20_16)
            * [2025](https://web.archive.org/web/20250506145311/https://www.bopa.ad/Documents/Detall?doc=QSH_2024_12_17_17_06_23)
        * Andorra la Vella:
            * [2020](https://web.archive.org/web/20250506142322/https://www.bopa.ad/Documents/Detall?doc=QAH20191128_08_58_08)
            * [2021](https://web.archive.org/web/20250506142457/https://www.bopa.ad/Documents/Detall?doc=QAH20201210_11_13_19)
            * [2022](https://web.archive.org/web/20250506142545/https://www.bopa.ad/Documents/Detall?doc=QAH20211111_16_06_47)
            * [2023](https://web.archive.org/web/20250506142748/https://www.bopa.ad/Documents/Detall?doc=QAH20221117_10_31_45)
            * [2024](https://web.archive.org/web/20250506142903/https://www.bopa.ad/Documents/Detall?doc=QAH_2023_11_17_12_40_42)
            * [2025](https://web.archive.org/web/20250506143036/https://www.bopa.ad/Documents/Detall?doc=QAH_2024_11_21_09_44_47)
        * Escaldes-Engordany:
            * [Parish foundation](https://web.archive.org/web/20130409081302/http://www.andorra.ad/ca-ES/Andorra/Pagines/comu_escaldes.aspx)
            * [2020](https://web.archive.org/web/20250506145337/https://www.bopa.ad/Documents/Detall?doc=QXH20200113_11_32_08)
            * [2021](https://web.archive.org/web/20250506145439/https://www.bopa.ad/Documents/Detall?doc=QXH20210127_13_33_13)
            * [2022](https://web.archive.org/web/20250506145442/https://www.bopa.ad/Documents/Detall?doc=QXH20220103_09_55_00)
            * [2023](https://web.archive.org/web/20250506145436/https://www.bopa.ad/Documents/Detall?doc=QXH20230102_11_21_25)
            * [2024](https://web.archive.org/web/20250506145940/https://www.bopa.ad/Documents/Detall?doc=QXH_2024_01_05_14_43_32)
            * [2025](https://web.archive.org/web/20250506145955/https://www.bopa.ad/Documents/Detall?doc=QXH_2024_12_23_11_47_04)
    """

    country = "AD"
    default_language = "ca"
    # The 1933 Revolution in Andorra
    start_year = 1934
    subdivisions = (
        "02",  # Canillo.
        "03",  # Encamp.
        "04",  # La Massana.
        "05",  # Ordino.
        "06",  # Sant Julià de Lòria.
        "07",  # Andorra la Vella.
        "08",  # Escaldes-Engordany.
    )
    subdivisions_aliases = {
        "Canillo": "02",
        "Encamp": "03",
        "La Massana": "04",
        "Ordino": "05",
        "Sant Julià de Lòria": "06",
        "Andorra la Vella": "07",
        "Escaldes-Engordany": "08",
    }
    supported_categories = (GOVERNMENT, PUBLIC)
    supported_languages = ("ca", "en_US", "uk")

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self) -> None:
        # New Year's Day.
        self._add_new_years_day(tr("Cap d'Any"))

        # Epiphany.
        self._add_epiphany_day(tr("Reis"))

        # Carnival.
        self._add_carnival_monday(tr("Carnaval"))

        if self._year >= 1994:
            # Constitution Day.
            self._add_holiday_mar_14(tr("Dia de la Constitució"))

        # Good Friday.
        self._add_good_friday(tr("Divendres Sant"))

        # Easter Monday.
        self._add_easter_monday(tr("Dilluns de Pasqua"))

        # Labor Day.
        self._add_labor_day(tr("Festa del treball"))

        # Whit Monday.
        self._add_whit_monday(tr("Dilluns de Pentecosta"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assumpció"))

        # Our Lady of Meritxell.
        self._add_holiday_sep_8(tr("Nostra Senyora de Meritxell"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Tots Sants"))

        # Immaculate Conception.
        self._add_immaculate_conception_day(tr("Immaculada Concepció"))

        # Christmas Day.
        self._add_christmas_day(tr("Nadal"))

        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Sant Esteve"))

    def _populate_subdiv_02_public_holidays(self):
        # Saint Roch's Day.
        self._add_holiday_aug_16(tr("Sant Roc"))

    def _populate_subdiv_03_public_holidays(self):
        # There are no holidays common to the whole parish.
        pass

    def _populate_subdiv_04_public_holidays(self):
        # Saint Anthony's Day.
        self._add_holiday_jan_17(tr("Sant Antoni"))

    def _populate_subdiv_05_public_holidays(self):
        # Saint Peter's Day.
        self._add_saints_peter_and_paul_day(tr("Sant Pere"))

    def _populate_subdiv_06_public_holidays(self):
        # Saint Julian's Day.
        self._add_holiday_jan_7(tr("Sant Julià"))

        # Virgin Mary of Canòlich.
        self._add_holiday_last_sat_of_may(tr("Diada de Canòlich"))

        # Sant Julià de Lòria Festival.
        name = tr("Festa Major de Sant Julià de Lòria")
        self._add_holiday_2_days_past_last_sat_of_jul(name)
        self._add_holiday_3_days_past_last_sat_of_jul(name)

    def _populate_subdiv_07_public_holidays(self):
        # Andorra la Vella Festival.
        name = tr("Festa Major d'Andorra la Vella")
        self._add_holiday_1st_sat_of_aug(name)
        self._add_holiday_1_day_past_1st_sat_of_aug(name)
        self._add_holiday_2_days_past_1st_sat_of_aug(name)

    def _populate_subdiv_08_public_holidays(self):
        # The parish of Escaldes-Engordany created on June 14, 1978.
        if self._year <= 1978:
            return None

        # Saint Michael of Engolasters' Day.
        self._add_holiday_may_7(tr("Sant Miquel d'Engolasters"))

        # Parish foundation day.
        name = tr("Diada de la creació de la parròquia")
        if self._year >= 1997:
            self._add_holiday_1st_sun_from_jun_14(name)
        else:
            self._add_holiday_jun_14(name)

        # Escaldes-Engordany Festival.
        name = tr("Festa Major d'Escaldes-Engordany")
        self._add_holiday_jul_25(name)
        self._add_holiday_jul_26(name)

    def _populate_government_holidays(self):
        # %s (from 1pm).
        begin_time_label = self.tr("%s (a partir de les 13h)")

        # Epiphany Eve.
        self._add_holiday_jan_5(begin_time_label % self.tr("Vigília de Reis"))

        # Maundy Thursday.
        self._add_holy_thursday(begin_time_label % self.tr("Dijous Sant"))

        # Christmas Eve.
        self._add_christmas_eve(begin_time_label % self.tr("Vigília de Nadal"))

        # New Year's Eve.
        self._add_new_years_eve(begin_time_label % self.tr("Vigília de Cap d'Any"))


class AD(Andorra):
    pass


class AND(Andorra):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.holiday_base import HolidayBase


class Gabon(
    HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Gabon holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Gabon>
        * [Decree 600/PR-MTPS of Jul 10, 1976](https://web.archive.org/web/20260307151111/https://journal-officiel.ga/14135-600-pr-mtps/)
        * [Decree 715/PR/MTEFPRH of Apr 2, 1992](https://web.archive.org/web/20260307150611/https://journal-officiel.ga/8555-715-pr-mtefprh/)
        * <https://web.archive.org/web/20200805200923/https://www.travail.gouv.ga/402-evenements/489-liste-des-jours-feries/>

    !!! note "Unavailable sources"
        * Decree 451/PR-MT.PS of Mar 14, 1970
        * Decree 147/PR of Dec 5, 1974
        * Decree 911/PR/MTE of Aug 25, 1980
        * Decree 00727/PR/MTEFP of Jun 29, 1998
        * Decree 000484/PR/MTE of May 26, 2004
    """

    country = "GA"
    default_language = "fr"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    # Decree 600/PR-MTPS of Jul 10, 1976.
    start_year = 1977
    supported_languages = ("en_US", "fr")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=GabonIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, cls=GabonStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Jour de l'an"))

        if self._year <= 1992:
            # Renovation Day.
            self._add_holiday_mar_12(tr("Journée de la rénovation"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Pâques"))

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        if self._year >= 2015:
            # Women's Rights Day.
            self._add_holiday_apr_17(tr("Journée des droits de la femme"))

        # Labor Day.
        self._add_labor_day(tr("Fête du Travail"))

        if self._year <= 1991:
            # Youth Day.
            self._add_holiday_2nd_sun_of_may(tr("Fête de la Jeunesse"))

        if self._year >= 2018:
            # Ascension Day.
            self._add_ascension_thursday(tr("Ascension"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Pentecôte"))

        # Whit Monday.
        self._add_whit_monday(tr("Lundi de Pentecôte"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assomption de Marie"))

        # Independence Day.
        self._add_holiday_aug_16(tr("Jour de l'indépendance"))

        # Independence Day Holiday.
        name = tr("Fête de l'indépendance")
        self._add_holiday_aug_17(name)

        if self._year <= 1991:
            self._add_holiday_aug_18(name)

        if self._year >= 2024:
            # Liberation Day.
            self._add_holiday_aug_30(tr("Journée de la Libération"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Toussaint"))

        # Christmas Day.
        self._add_christmas_day(tr("Noël"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Fin du Ramadan"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Fête du sacrifice"))


class GA(Gabon):
    pass


class GAB(Gabon):
    pass


class GabonIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2001: (MAR, 6),
        2002: (FEB, 23),
        2003: (FEB, 12),
        2004: (FEB, 2),
        2008: (DEC, 9),
        2009: (NOV, 28),
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_FITR_DATES = {
        2001: (DEC, 17),
        2002: (DEC, 6),
        2003: (NOV, 26),
        2005: (NOV, 4),
        2006: (OCT, 24),
        2008: (OCT, 2),
        2009: (SEP, 21),
        2011: (AUG, 31),
        2014: (JUL, 29),
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
    }


class GabonStaticHolidays:
    """Gabon special holidays.

    References:
        * <https://web.archive.org/web/20241116171700/https://info241.com/referendum-constitutionnel-du-16-novembre-trois-jours-feries,9646>
        * <https://web.archive.org/web/20250409074948/https://info241.com/presidentielle-2025-les-11-et-12-avril-declares-feries-pour,10205>
        * <https://web.archive.org/web/20250501091256/https://info241.com/gabon-le-2-mai-declare-ferie-et-recuperable-pour-l-investiture,2385>
        * <https://web.archive.org/web/20260307182549/https://info241.com/gabon-le-samedi-27-septembre-declare-jour-ferie-pour-les,2558>
    """

    # Public holiday.
    public_holiday = tr("Jour férié")

    special_public_holidays = {
        2024: (
            (NOV, 14, public_holiday),
            (NOV, 15, public_holiday),
            (NOV, 16, public_holiday),
        ),
        2025: (
            (APR, 11, public_holiday),
            (APR, 12, public_holiday),
            (MAY, 2, public_holiday),
            (SEP, 27, public_holiday),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAY, JUN, OCT, NOV
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Bermuda(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Bermuda holidays.

    References:
        * [Public Holidays Act 1947](https://web.archive.org/web/20250527163956/https://www.bermudalaws.bm/Laws/Consolidated%20Law/1947/Public%20Holidays%20Act%201947)
        * [Public Holidays Amendment Act 1999](https://web.archive.org/web/20250527163749/https://www.bermudalaws.bm/Laws/Annual%20Law/Acts/1999/Public%20Holidays%20Amendment%20Act%201999)
        * [Public Holidays Amendment Act 2008](https://web.archive.org/web/20250527163810/https://www.bermudalaws.bm/Laws/Annual%20Law/Acts/2008/Public%20Holidays%20Amendment%20Act%202008)
        * [Public Holidays Amendment (No. 2) Act 2009](https://web.archive.org/web/20250527163816/https://www.bermudalaws.bm/Laws/Annual%20Law/Acts/2009/Public%20Holidays%20Amendment%20(No.%202)%20Act%202009)
        * [Public Holidays Amendment Act 2017](https://web.archive.org/web/20250527163819/https://www.bermudalaws.bm/Laws/Annual%20Law/Acts/2017/Public%20Holidays%20Amendment%20Act%202017)
        * [Public Holidays Amendment Act 2020](https://web.archive.org/web/20250527163836/https://www.bermudalaws.bm/Laws/Annual%20Law/Acts/2020/Public%20Holidays%20Amendment%20Act%202020)
        * [Government of Bermuda](https://web.archive.org/web/20250530055854/https://www.gov.bm/public-holidays)
        * [Proclamation BR 149 / 2018](https://web.archive.org/web/20250530114331/https://www.bermudalaws.bm/Laws/Annual%20Law/Statutory%20Instruments/2018/Proclamation%20-%20Bermuda%20Day%20Public%20Holiday)
    """

    country = "BM"
    default_language = "en_BM"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_BM", "en_US")
    start_year = 1948

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, BermudaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Bermuda Day.
        name = tr("Bermuda Day")
        if self._year <= 2017:
            self._add_observed(self._add_holiday_may_24(name))
        elif self._year == 2019:
            self._add_holiday_may_24(name)
        elif self._year <= 2020:
            self._add_holiday_last_fri_of_may(name)
        else:
            self._add_holiday_3_days_prior_last_mon_of_may(name)

        # Queen's Birthday.
        name = tr("Queen's Birthday")
        if self._year <= 1999:
            self._add_holiday_3rd_mon_of_jun(name)
        elif self._year <= 2008:
            self._add_holiday_2_days_past_2nd_sat_of_jun(name)

        # National Heroes Day.
        name = tr("National Heroes Day")
        if self._year == 2008:
            self._add_holiday_2nd_mon_of_oct(name)
        elif self._year >= 2009:
            self._add_holiday_3rd_mon_of_jun(name)

        self._add_holiday_4_days_prior_1st_mon_of_aug(
            # Cup Match Day.
            tr("Cup Match Day")
            if self._year <= 1999
            # Emancipation Day.
            else tr("Emancipation Day")
        )

        self._add_holiday_3_days_prior_1st_mon_of_aug(
            # Somers Day.
            tr("Somers Day")
            if self._year <= 2019
            # Mary Prince Day.
            else tr("Mary Prince Day")
        )

        # Labor Day.
        self._add_holiday_1st_mon_of_sep(tr("Labour Day"))

        # Remembrance Day.
        self._add_observed(self._add_remembrance_day(tr("Remembrance Day")))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class BM(Bermuda):
    pass


class BMU(Bermuda):
    pass


class BermudaStaticHolidays:
    """Bermuda special holidays.

    References:
        * [June 5, 2007 Holiday](https://web.archive.org/web/20250530122818/https://www.bermudalaws.bm/Laws/Annual%20Law/Statutory%20Instruments/2007/Proclamation)
        * Portuguese Welcome 170th Anniversary
            * <https://web.archive.org/web/20241210201648/https://www.royalgazette.com/other/news/article/20191025/portuguese-welcome-170th-anniversary-holiday/>
            * <https://web.archive.org/web/20250530122008/https://www.bermudalaws.bm/Laws/Annual%20Law/Statutory%20Instruments/2018/Proclamation%20-%20Portguese%20Public%20Holiday>
        * Flora Duffy Day
            * <https://web.archive.org/web/20240613083451/https://bernews.com/2021/10/governor-signs-flora-duffy-day-proclamation/>
            * <https://web.archive.org/web/20250530113359/https://www.bermudalaws.bm/Laws/Annual%20Law/Statutory%20Instruments/2021/Proclamation%20for%2018%20October%202021%20Holiday>
        * The Coronation of His Majesty King Charles III Holiday
            * <https://web.archive.org/web/20250222035055/https://gov.bm/articles/coronation-his-majesty-king-charles-iii-her-majesty-queen-consort>
            * <https://web.archive.org/web/20250530112129/https://www.bermudalaws.bm/Laws/Annual%20Law/Statutory%20Instruments/2023/Proclamation%20for%208%20May%202023%20Holiday>
    """

    special_public_holidays = {
        # Public Holiday.
        2007: (JUN, 5, tr("Public Holiday")),
        # Portuguese Welcome 170th Anniversary.
        2019: (NOV, 4, tr("Portuguese Welcome 170th Anniversary")),
        # Flora Duffy Day.
        2021: (OCT, 18, tr("Flora Duffy Day")),
        # The Coronation of His Majesty King Charles III.
        2023: (MAY, 8, tr("The Coronation of His Majesty King Charles III")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAY, JUN, OCT, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_TO_PREV_FRI,
    SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class BritishVirginIslands(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """British Virgin Islands holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_British_Virgin_Islands>
        * [Public Holidays (Amendment) Act, 2021](https://web.archive.org/web/20250522154946/https://laws.gov.vg/Laws/public-holidays-amendment-act-2021)
        * <https://web.archive.org/web/20250401075439/https://www.timeanddate.com/holidays/british-virgin-islands/>
        * [2015](https://web.archive.org/web/20250317055722/https://www.bvi.gov.vg/media-centre/2015-public-holidays)
        * [2016](https://web.archive.org/web/20250323113905/https://www.bvi.gov.vg/media-centre/2016-public-holidays)
        * [2017](https://web.archive.org/web/20250515155409/https://bvi.gov.vg/media-centre/2017-public-holidays)
        * [2018](https://web.archive.org/web/20250318095807/https://www.bvi.gov.vg/media-centre/2018-public-holidays)
        * [2019](https://web.archive.org/web/20250315162108/https://www.bvi.gov.vg/media-centre/2019-public-holidays)
        * [2020](https://web.archive.org/web/20250111172241/https://www.bvi.gov.vg/media-centre/2020-public-holidays)
        * [2021](https://web.archive.org/web/20250417222246/http://www.bvi.gov.vg/media-centre/2021-public-holidays-revised)
        * [2022](https://web.archive.org/web/20250420005523/https://www.bvi.gov.vg/media-centre/revised-2022-public-holidays)
        * [2023](https://web.archive.org/web/20250111170511/https://bvi.gov.vg/media-centre/updatedofficialholidays20231)
        * [2024](https://web.archive.org/web/20250502194703/https://www.bvi.gov.vg/media-centre/cabinet-approves-2024-public-holidays)
        * [2025](https://web.archive.org/web/20250124122752/https://bvi.gov.vg/media-centre/cabinet-approves-2025-holidays)
    """

    country = "VG"
    default_language = "en_VG"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_US", "en_VG")
    start_year = 1967

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, BritishVirginIslandsStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 2000)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        self._add_holiday_1st_mon_before_mar_7(
            # Lavity Stoutt's Birthday.
            tr("The Anniversary of the Birth of Hamilton Lavity Stoutt")
        )

        if self._year <= 2020:
            # Commonwealth Day.
            self._add_holiday_2nd_mon_of_mar(tr("Commonwealth Day"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Whit Monday.
        self._add_whit_monday(tr("Whit Monday"))

        # Sovereign's Birthday.
        name = tr("Sovereign's Birthday")
        if self._year >= 2019:
            sovereign_birthday_dates = {
                2019: (JUN, 7),
                2023: (JUN, 16),
            }
            if dt := sovereign_birthday_dates.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_holiday_2nd_fri_of_jun(name)
        else:
            self._add_holiday_2nd_sat_of_jun(name)

        name = (
            # Virgin Islands Day.
            tr("Virgin Islands Day")
            if self._year >= 2021
            # Territory Day.
            else tr("Territory Day")
            if self._year >= 1978
            # Colony Day.
            else tr("Colony Day")
        )
        territory_day_dates = {
            2015: (JUN, 29),
            2020: (JUN, 29),
        }
        if dt := territory_day_dates.get(self._year):
            self._add_holiday(name, dt)
        else:
            if self._year >= 2021:
                self._add_holiday_1st_mon_of_jul(name)
            else:
                self._add_observed(
                    self._add_holiday_jul_1(name), rule=SAT_TO_PREV_FRI + SUN_TO_NEXT_MON
                )

        self._add_holiday_1st_mon_of_aug(
            # Emancipation Monday.
            tr("Emancipation Monday")
            if self._year >= 2021
            # Festival Monday.
            else tr("Festival Monday")
        )

        self._add_holiday_1_day_past_1st_mon_of_aug(
            # Emancipation Tuesday.
            tr("Emancipation Tuesday")
            if self._year >= 2021
            # Festival Tuesday.
            else tr("Festival Tuesday")
        )

        self._add_holiday_2_days_past_1st_mon_of_aug(
            # Emancipation Wednesday.
            tr("Emancipation Wednesday")
            if self._year >= 2021
            # Festival Wednesday.
            else tr("Festival Wednesday")
        )

        if self._year <= 2020:
            # Saint Ursula's Day.
            name = tr("Saint Ursula's Day")
            saint_ursula_dates = {
                2015: (OCT, 19),
                2020: (OCT, 23),
            }
            if dt := saint_ursula_dates.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_observed(
                    self._add_holiday_oct_21(name), rule=SAT_TO_PREV_FRI + SUN_TO_NEXT_MON
                )

        if self._year >= 2021:
            # Heroes and Foreparents Day.
            self._add_holiday_3rd_mon_of_oct(tr("Heroes and Foreparents Day"))

            # The Great March of 1949 and Restoration Day.
            self._add_holiday_4th_mon_of_nov(tr("The Great March of 1949 and Restoration Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class VG(BritishVirginIslands):
    pass


class VGB(BritishVirginIslands):
    pass


class BritishVirginIslandsStaticHolidays:
    """British Virgin Islands special holidays.

    References:
        * [2019](https://web.archive.org/web/20250417134346/https://www.bvi.gov.vg/media-centre/public-holiday-be-observed-funeral-late-honourable-ralph-t-o-neal-obe)
        * [2022](https://web.archive.org/web/20250315174707/https://www.bvi.gov.vg/media-centre/public-holiday-observe-her-majesty-s-platinum-jubilee)
        * [2023](https://web.archive.org/web/20250505073654/https://bvi.gov.vg/media-centre/may-8-and-june-16-declared-public-holidays)
    """

    special_public_holidays = {
        # State Funeral of Honourable Ralph T. O'Neal.
        2019: (DEC, 11, tr("State Funeral of Honourable Ralph T. O'Neal")),
        # Platinum Jubilee of Elizabeth II.
        2022: (JUN, 3, tr("Queen Elizabeth II's Platinum Jubilee")),
        # Coronation of Charles III.
        2023: (MAY, 8, tr("The King's Coronation")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Czechia(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Czechia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Czech_Republic>
        * [Law 93/1951](https://web.archive.org/web/20240924164547/https://www.zakonyprolidi.cz/cs/1951-93)
        * [Law 204/1990 (Jan Hus Day)](https://web.archive.org/web/20240924164541/https://www.zakonyprolidi.cz/cs/1990-204)
        * [Law 245/2000](https://web.archive.org/web/20241215175816/https://www.zakonyprolidi.cz/cs/2000-245)
    """

    country = "CZ"
    default_language = "cs"
    supported_languages = ("cs", "en_US", "sk", "uk")
    # Law 93/1951.
    start_year = 1952

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nový rok"))

        if self._year >= 2000:
            # Independent Czech State Restoration Day.
            self._add_holiday_jan_1(tr("Den obnovy samostatného českého státu"))

        if self._year >= 2016:
            # Good Friday.
            self._add_good_friday(tr("Velký pátek"))

        # Easter Monday.
        self._add_easter_monday(tr("Velikonoční pondělí"))

        # Labor Day.
        self._add_labor_day(tr("Svátek práce"))

        if self._year >= 2004:
            # Victory Day.
            name = tr("Den vítězství")
        elif self._year >= 2001:
            # Liberation Day.
            name = tr("Den osvobození")
        else:
            # Day of liberation from Fascism.
            name = tr("Den osvobození od fašismu")

        self._add_world_war_two_victory_day(name, is_western=(self._year >= 1992))

        if self._year >= 1990:
            # Saints Cyril and Methodius Day.
            self._add_holiday_jul_5(tr("Den slovanských věrozvěstů Cyrila a Metoděje"))

            # Jan Hus Day.
            self._add_holiday_jul_6(tr("Den upálení mistra Jana Husa"))

        if self._year >= 2000:
            # Statehood Day.
            self._add_holiday_sep_28(tr("Den české státnosti"))

        # Independent Czechoslovak State Day.
        self._add_holiday_oct_28(tr("Den vzniku samostatného československého státu"))

        if self._year >= 1990:
            self._add_holiday_nov_17(
                # Struggle for Freedom and Democracy Day and International Students' Day.
                tr("Den boje za svobodu a demokracii a Mezinárodní den studentstva")
                if self._year >= 2019
                # Struggle for Freedom and Democracy Day.
                else tr("Den boje za svobodu a demokracii")
            )

            # Christmas Eve.
            self._add_christmas_eve(tr("Štědrý den"))

        # Christmas Day.
        self._add_christmas_day(tr("1. svátek vánoční"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("2. svátek vánoční"))


class CZ(Czechia):
    pass


class CZE(Czechia):
    pass
